@@ -18,6 +18,7 @@ package org.apache.pdfbox.debugger;
 
 import java.awt.BorderLayout;
 import java.awt.Component;
+import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.FileDialog;
 import java.awt.Toolkit;
@@ -42,7 +43,7 @@ import java.util.Arrays;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.StringTokenizer;
+import javax.imageio.spi.IIORegistry;
 import javax.print.attribute.HashPrintRequestAttributeSet;
 import javax.print.attribute.PrintRequestAttributeSet;
 import javax.print.attribute.standard.Sides;
@@ -105,11 +106,11 @@ import org.apache.pdfbox.debugger.ui.RecentFiles;
 import org.apache.pdfbox.debugger.ui.RotationMenu;
 import org.apache.pdfbox.debugger.ui.Tree;
 import org.apache.pdfbox.debugger.ui.ZoomMenu;
+import org.apache.pdfbox.filter.FilterFactory;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.common.PDPageLabels;
 import org.apache.pdfbox.pdmodel.encryption.InvalidPasswordException;
-import org.apache.pdfbox.pdmodel.font.PDFont;
 import org.apache.pdfbox.pdmodel.font.PDType1Font;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceCMYK;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceRGB;
@@ -300,10 +301,20 @@ public class PDFDebugger extends JFrame
             }
         });
 
+        initGlobalEventHandlers();
+    }
+
+    /**
+     * Initialize application global event handlers. Protected to allow
+     * subclasses to override this method if they don't want the global event
+     * handler overridden.
+     */
+    @SuppressWarnings("WeakerAccess")
+    protected void initGlobalEventHandlers()
+    {
         // Mac OS X file open/quit handler
-        if (IS_MAC_OS && !isMinJdk9())
+        if (IS_MAC_OS)
         {
-            //TODO this needs to be rewritten for JDK9, see PDFBOX-4013
             try
             {
                 Method osxOpenFiles = getClass().getDeclaredMethod("osxOpenFiles", String.class);
@@ -415,11 +426,8 @@ public class PDFDebugger extends JFrame
             }
         });
 
-        if (!IS_MAC_OS)
-        {
-            fileMenu.addSeparator();
-            fileMenu.add(printMenuItem);
-        }
+        fileMenu.addSeparator();
+        fileMenu.add(printMenuItem);
 
         JMenuItem exitMenuItem = new JMenuItem("Exit");
         exitMenuItem.setAccelerator(KeyStroke.getKeyStroke("alt F4"));
@@ -630,7 +638,7 @@ public class PDFDebugger extends JFrame
                 openDialog.setVisible(true);
                 if (openDialog.getFile() != null)
                 {
-                    readPDFFile(openDialog.getFile(), "");
+                    readPDFFile(new File(openDialog.getDirectory(),openDialog.getFile()), "");
                 }
             }
             else
@@ -898,7 +906,9 @@ public class PDFDebugger extends JFrame
         {
             selectedNode = ((MapEntry)selectedNode).getKey();
             selectedNode = getUnderneathObject(selectedNode);
-            FlagBitsPane flagBitsPane = new FlagBitsPane((COSDictionary) parentNode, (COSName) selectedNode);
+            FlagBitsPane flagBitsPane = new FlagBitsPane(document,
+                    (COSDictionary) parentNode,
+                    (COSName) selectedNode);
             replaceRightComponent(flagBitsPane.getPane());
         }
     }
@@ -1075,7 +1085,7 @@ public class PDFDebugger extends JFrame
         return data;
     }
 
-    private void exitMenuItemActionPerformed(ActionEvent evt)
+    private void exitMenuItemActionPerformed(ActionEvent ignored)
     {
         if( document != null )
         {
@@ -1093,6 +1103,16 @@ public class PDFDebugger extends JFrame
                 throw new RuntimeException(e);
             }
         }
+        performApplicationExit();
+    }
+
+    /**
+     * Exit the application after the window is closed. This is protected to let
+     * subclasses override the behavior.
+     */
+    @SuppressWarnings("WeakerAccess")
+    protected void performApplicationExit()
+    {
         System.exit(0);
     }
 
@@ -1127,7 +1147,15 @@ public class PDFDebugger extends JFrame
             }
             if (job.printDialog(pras))
             {
-                job.print(pras);
+                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
+                try
+                {
+                    job.print(pras);
+                }
+                finally
+                {
+                    setCursor(Cursor.getDefaultCursor());
+                }
             }
         }
         catch (PrinterException e)
@@ -1141,23 +1169,7 @@ public class PDFDebugger extends JFrame
      */
     private void exitForm(WindowEvent evt)
     {
-        if( document != null )
-        {
-            try
-            {
-                document.close();
-                if (!currentFilePath.startsWith("http"))
-                {
-                    recentFiles.addFile(currentFilePath);
-                }
-                recentFiles.close();
-            }
-            catch( IOException e )
-            {
-                throw new RuntimeException(e);
-            }
-        }
-        System.exit(0);
+        exitMenuItemActionPerformed(null);
     }
 
     /**
@@ -1202,6 +1214,8 @@ public class PDFDebugger extends JFrame
             // Yes this is always true
             PDDeviceCMYK.INSTANCE.toRGB(new float[] { 0, 0, 0, 0} );
             PDDeviceRGB.INSTANCE.toRGB(new float[] { 0, 0, 0 } );
+            IIORegistry.getDefaultInstance();
+            FilterFactory.INSTANCE.getFilter(COSName.FLATE_DECODE);
         }
 
         // open file, if any
@@ -1249,7 +1263,7 @@ public class PDFDebugger extends JFrame
         readPDFFile(file, password);
     }
     
-    private void readPDFFile(File file, String password) throws IOException
+    private void readPDFFile(final File file, String password) throws IOException
     {
         if( document != null )
         {
@@ -1261,7 +1275,17 @@ public class PDFDebugger extends JFrame
         }
         currentFilePath = file.getPath();
         recentFiles.removeFile(file.getPath());
-        parseDocument( file, password );
+        DocumentOpener documentOpener = new DocumentOpener(password)
+        {
+            @Override
+            PDDocument open() throws IOException
+            {
+                return PDDocument.load(file, password);
+            }
+        };
+        document = documentOpener.parse();
+        printMenuItem.setEnabled(true);
+        reopenMenuItem.setEnabled(true);
         
         initTree();
         
@@ -1277,7 +1301,7 @@ public class PDFDebugger extends JFrame
         addRecentFileItems();
     }
     
-    private void readPDFurl(String urlString, String password) throws IOException
+    private void readPDFurl(final String urlString, String password) throws IOException
     {
         if (document != null)
         {
@@ -1288,8 +1312,15 @@ public class PDFDebugger extends JFrame
             }
         }
         currentFilePath = urlString;
-        URL url = new URL(urlString);
-        document = PDDocument.load(url.openStream(), password);
+        DocumentOpener documentOpener = new DocumentOpener(password)
+        {
+            @Override
+            PDDocument open() throws IOException
+            {
+                return PDDocument.load(new URL(urlString).openStream(), password);
+            }
+        };
+        document = documentOpener.parse();
         printMenuItem.setEnabled(true);
         reopenMenuItem.setEnabled(true);
 
@@ -1327,45 +1358,65 @@ public class PDFDebugger extends JFrame
             tree.setSelectionPath(treeStatus.getPathForString("Root"));
         }
     }
-    
+
     /**
-     * This will parse a document.
-     *
-     * @param file The file addressing the document.
-     *
-     * @throws IOException If there is an error parsing the document.
+     * Internal class to avoid double code in password entry loop.
      */
-    private void parseDocument( File file, String password )throws IOException
+    abstract class DocumentOpener
     {
-        while (true)
+        String password;
+
+        DocumentOpener(String password)
         {
-            try
+            this.password = password;
+        }
+
+        /**
+         * Override to load the actual input type (File, URL, stream), don't call it directly!
+         * 
+         * @return
+         * @throws IOException 
+         */
+        abstract PDDocument open() throws IOException;
+
+        /**
+         * Call this!
+         * 
+         * @return
+         * @throws IOException 
+         */
+        final PDDocument parse() throws IOException 
+        {
+            while (true)
             {
-                document = PDDocument.load(file, password);
-            }
-            catch (InvalidPasswordException ipe)
-            {
-                // https://stackoverflow.com/questions/8881213/joptionpane-to-get-password
-                JPanel panel = new JPanel();
-                JLabel label = new JLabel("Password:");
-                JPasswordField pass = new JPasswordField(10);
-                panel.add(label);
-                panel.add(pass);
-                String[] options = new String[] {"OK", "Cancel"};
-                int option = JOptionPane.showOptionDialog(null, panel, "Enter password",
-                         JOptionPane.NO_OPTION, JOptionPane.PLAIN_MESSAGE,
-                         null, options, "");
-                if (option == 0)
+                try
                 {
-                    password = new String(pass.getPassword());
-                    continue;
+                    return open();
+                }
+                catch (InvalidPasswordException ipe)
+                {
+                    // https://stackoverflow.com/questions/8881213/joptionpane-to-get-password
+                    JPanel panel = new JPanel();
+                    JLabel label = new JLabel("Password:");
+                    JPasswordField pass = new JPasswordField(10);
+                    panel.add(label);
+                    panel.add(pass);
+                    String[] options = new String[]
+                    {
+                        "OK", "Cancel"
+                    };
+                    int option = JOptionPane.showOptionDialog(null, panel, "Enter password",
+                            JOptionPane.NO_OPTION, JOptionPane.PLAIN_MESSAGE,
+                            null, options, "");
+                    if (option == 0)
+                    {
+                        password = new String(pass.getPassword());
+                        continue;
+                    }
+                    throw ipe;
                 }
-                throw ipe;
             }
-            break;
-        }        
-        printMenuItem.setEnabled(true);
-        reopenMenuItem.setEnabled(true);
+        }
     }
 
     private void addRecentFileItems()
@@ -1446,26 +1497,4 @@ public class PDFDebugger extends JFrame
         }
         return null;
     }
-
-    private static boolean isMinJdk9()
-    {
-        // strategy from lucene-solr/lucene/core/src/java/org/apache/lucene/util/Constants.java
-        String version = System.getProperty("java.specification.version");
-        final StringTokenizer st = new StringTokenizer(version, ".");
-        try
-        {
-            int major = Integer.parseInt(st.nextToken());
-            int minor = 0;
-            if (st.hasMoreTokens())
-            {
-                minor = Integer.parseInt(st.nextToken());
-            }
-            return major > 1 || (major == 1 && minor >= 9);
-        }
-        catch (NumberFormatException nfe)
-        {
-            // maybe some new numbering scheme in the 22nd century
-            return true;
-        }
-    }
 }
