@@ -28,6 +28,8 @@ import java.awt.event.MouseMotionListener;
 import java.awt.geom.AffineTransform;
 import java.awt.image.BufferedImage;
 import java.io.IOException;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.concurrent.ExecutionException;
 import javax.swing.BoxLayout;
 import javax.swing.JLabel;
@@ -44,6 +46,10 @@ import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.rendering.PDFRenderer;
 import org.apache.pdfbox.debugger.PDFDebugger;
 import org.apache.pdfbox.debugger.ui.HighResolutionImageIcon;
+import org.apache.pdfbox.pdmodel.common.PDRectangle;
+import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotationWidget;
+import org.apache.pdfbox.pdmodel.interactive.form.PDAcroForm;
+import org.apache.pdfbox.pdmodel.interactive.form.PDField;
 
 /**
  * Display the page number and a page rendering.
@@ -61,6 +67,10 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
     private RotationMenu rotationMenu;
     private final JLabel statuslabel;
     private final PDPage page;
+    private String labelText = "";
+    private final Map<PDRectangle, String> rectMap = new HashMap<PDRectangle, String>();
+    private final AffineTransform defaultTransform = GraphicsEnvironment.getLocalGraphicsEnvironment().
+                        getDefaultScreenDevice().getDefaultConfiguration().getDefaultTransform();
 
     public PagePane(PDDocument document, COSDictionary pageDict, JLabel statuslabel)
     {
@@ -69,6 +79,27 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
         this.document = document;
         this.statuslabel = statuslabel;
         initUI();
+        initRectMap();
+    }
+
+    private void initRectMap()
+    {
+        PDAcroForm acroForm = document.getDocumentCatalog().getAcroForm();
+        if (acroForm == null)
+        {
+            return;
+        }
+        for (PDField field : acroForm.getFieldTree())
+        {
+            String fullyQualifiedName = field.getFullyQualifiedName();
+            for (PDAnnotationWidget widget : field.getWidgets())
+            {
+                if (page.equals(widget.getPage()))
+                {
+                    rectMap.put(widget.getRectangle(), fullyQualifiedName);
+                }
+            }
+        }
     }
 
     private void initUI()
@@ -185,8 +216,8 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
         float offsetX = page.getCropBox().getLowerLeftX();
         float offsetY = page.getCropBox().getLowerLeftY();
         float zoomScale = zoomMenu.getPageZoomScale();
-        float x = e.getX() / zoomScale;
-        float y = e.getY() / zoomScale;
+        float x = e.getX() / zoomScale * (float) defaultTransform.getScaleX();
+        float y = e.getY() / zoomScale * (float) defaultTransform.getScaleY();
         int x1, y1;
         switch ((RotationMenu.getRotationDegrees() + page.getRotation()) % 360)
         {
@@ -208,7 +239,19 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
                 y1 = (int) (height - y + offsetY);
                 break;
         }
-        statuslabel.setText(x1 + "," + y1);
+        String text = "x: " + x1 + ", y: " + y1;
+
+        // are we in a field widget?
+        for (Map.Entry<PDRectangle, String> entry : rectMap.entrySet())
+        {
+            if (entry.getKey().contains(x1, y1))
+            {
+                text += ", field: " + rectMap.get(entry.getKey());
+                break;
+            }
+        }
+
+        statuslabel.setText(text);
     }
 
     @Override
@@ -234,7 +277,7 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
     @Override
     public void mouseExited(MouseEvent e)
     {
-        statuslabel.setText("");
+        statuslabel.setText(labelText);
     }
 
     /**
@@ -257,14 +300,16 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
         protected BufferedImage doInBackground() throws IOException
         {
             label.setIcon(null);
-            label.setText("Rendering...");
+            labelText = "Rendering...";
+            label.setText(labelText);
             PDFRenderer renderer = new PDFRenderer(document);
             renderer.setSubsamplingAllowed(allowSubsampling);
             long t0 = System.currentTimeMillis();
-            statuslabel.setText("Rendering...");
+            statuslabel.setText(labelText);
             BufferedImage bim = renderer.renderImage(pageIndex, scale);
             float t = (System.currentTimeMillis() - t0) / 1000f;
-            statuslabel.setText("Rendered in " + t + " second" + (t > 1 ? "s" : ""));
+            labelText = "Rendered in " + t + " second" + (t > 1 ? "s" : "");
+            statuslabel.setText(labelText);
             return ImageUtil.getRotatedImage(bim, rotation);
         }
 
@@ -280,10 +325,8 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
                 // a smaller size than the image to compensate that the
                 // image is scaled up with some screen configurations (e.g. 125% on windows).
                 // See PDFBOX-3665 for more sample code and discussion.
-                AffineTransform tx = GraphicsEnvironment.getLocalGraphicsEnvironment().
-                        getDefaultScreenDevice().getDefaultConfiguration().getDefaultTransform();
-                label.setSize((int) Math.ceil(image.getWidth() / tx.getScaleX()), 
-                              (int) Math.ceil(image.getHeight() / tx.getScaleY()));
+                label.setSize((int) Math.ceil(image.getWidth() / defaultTransform.getScaleX()), 
+                              (int) Math.ceil(image.getHeight() / defaultTransform.getScaleY()));
                 label.setIcon(new HighResolutionImageIcon(image, label.getWidth(), label.getHeight()));
                 label.setText(null);
             }
