@@ -20,56 +20,71 @@ import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
-import java.security.InvalidKeyException;
+import java.security.GeneralSecurityException;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.PublicKey;
-import java.security.SignatureException;
+import java.security.Security;
 import java.security.cert.Certificate;
 import java.security.cert.CertificateException;
+import java.security.cert.CertificateExpiredException;
 import java.security.cert.CertificateFactory;
+import java.security.cert.CertificateNotYetValidException;
 import java.security.cert.X509Certificate;
 import java.text.SimpleDateFormat;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.Date;
+import java.util.HashSet;
+import java.util.Set;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
-
 import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSInputStream;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.cos.COSString;
+import org.apache.pdfbox.examples.signature.cert.CertificateVerificationException;
+import org.apache.pdfbox.examples.signature.cert.CertificateVerifier;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
+import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
 import org.apache.pdfbox.util.Hex;
+import org.bouncycastle.asn1.ASN1Object;
+import org.bouncycastle.asn1.cms.Attribute;
+import org.bouncycastle.asn1.cms.AttributeTable;
+import org.bouncycastle.asn1.cms.CMSAttributes;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.x509.Time;
 import org.bouncycastle.cert.X509CertificateHolder;
+import org.bouncycastle.cert.jcajce.JcaCertStore;
 import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
 import org.bouncycastle.cms.CMSException;
 import org.bouncycastle.cms.CMSProcessable;
 import org.bouncycastle.cms.CMSProcessableByteArray;
 import org.bouncycastle.cms.CMSSignedData;
 import org.bouncycastle.cms.SignerInformation;
+import org.bouncycastle.cms.SignerInformationVerifier;
 import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.tsp.TSPException;
 import org.bouncycastle.tsp.TimeStampToken;
 import org.bouncycastle.util.Selector;
 import org.bouncycastle.util.Store;
-import org.bouncycastle.util.StoreException;
 
 /**
- * This will read a document from the filesystem, decrypt it and do something with the signature.
+ * This will get the signature(s) from the document, do some verifications and
+ * show the signature(s) and the certificates. This is a complex topic - the
+ * code here is an example and not a production-ready solution.
  *
  * @author Ben Litchfield
  */
 public final class ShowSignature
 {
     private final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
-    
+
     private ShowSignature()
     {
     }
@@ -80,24 +95,22 @@ public final class ShowSignature
      * @param args The command-line arguments.
      *
      * @throws IOException If there is an error reading the file.
-     * @throws CertificateException
-     * @throws java.security.NoSuchAlgorithmException
-     * @throws java.security.NoSuchProviderException
      * @throws org.bouncycastle.tsp.TSPException
+     * @throws java.security.GeneralSecurityException
+     * @throws org.apache.pdfbox.examples.signature.cert.CertificateVerificationException
      */
-    public static void main(String[] args) throws IOException, CertificateException,
-                                                  NoSuchAlgorithmException,
-                                                  NoSuchProviderException,
-                                                  TSPException
+    public static void main(String[] args) throws IOException, TSPException, GeneralSecurityException,
+            CertificateVerificationException
     {
+        // register BouncyCastle provider, needed for "exotic" algorithms
+        Security.addProvider(SecurityProvider.getProvider());
+
         ShowSignature show = new ShowSignature();
         show.showSignature( args );
     }
 
-    private void showSignature(String[] args) throws IOException, CertificateException,
-                                                     NoSuchAlgorithmException,
-                                                     NoSuchProviderException,
-                                                     TSPException
+    private void showSignature(String[] args) throws IOException, TSPException, GeneralSecurityException,
+            CertificateVerificationException
     {
         if( args.length != 2 )
         {
@@ -169,8 +182,6 @@ public final class ShowSignature
                             subFilter.equals("ETSI.CAdES.detached"))
                         {
                             verifyPKCS7(buf, contents, sig);
-                            
-                            //TODO check certificate chain, revocation lists, timestamp...
                         }
                         else if (subFilter.equals("adbe.pkcs7.sha1"))
                         {
@@ -183,13 +194,12 @@ public final class ShowSignature
 
                             byte[] hash = MessageDigest.getInstance("SHA1").digest(buf);
                             verifyPKCS7(hash, contents, sig);
-                            
-                            //TODO check certificate chain, revocation lists, timestamp...
                         }
                         else if (subFilter.equals("adbe.x509.rsa_sha1"))
                         {
                             // example: PDFBOX-2693.pdf
                             COSString certString = (COSString) sigDict.getDictionaryObject(COSName.CERT);
+                            //TODO this could also be an array.
                             if (certString == null)
                             {
                                 System.err.println("The /Cert certificate string is missing in the signature dictionary");
@@ -200,21 +210,52 @@ public final class ShowSignature
                             ByteArrayInputStream certStream = new ByteArrayInputStream(certData);
                             Collection<? extends Certificate> certs = factory.generateCertificates(certStream);
                             System.out.println("certs=" + certs);
-                            
-                            //TODO verify signature
+
+                            X509Certificate cert = (X509Certificate) certs.iterator().next();
+
+                            // to verify signature, see code at
+                            // https://stackoverflow.com/questions/43383859/
+
+                            try
+                            {
+                                if (sig.getSignDate() != null)
+                                {
+                                    cert.checkValidity(sig.getSignDate().getTime());
+                                    System.out.println("Certificate valid at signing time");
+                                }
+                                else
+                                {
+                                    System.err.println("Certificate cannot be verified without signing time");
+                                }
+                            }
+                            catch (CertificateExpiredException ex)
+                            {
+                                System.err.println("Certificate expired at signing time");
+                            }
+                            catch (CertificateNotYetValidException ex)
+                            {
+                                System.err.println("Certificate not yet valid at signing time");
+                            }
+                            if (CertificateVerifier.isSelfSigned(cert))
+                            {
+                                System.err.println("Certificate is self-signed, LOL!");
+                            }
+                            else
+                            {
+                                System.out.println("Certificate is not self-signed");
+
+                                if (sig.getSignDate() != null)
+                                {
+                                    verifyCertificateChain(new JcaCertStore(certs),
+                                            cert,
+                                            sig.getSignDate().getTime());
+                                }
+                            }
                         }
                         else if (subFilter.equals("ETSI.RFC3161"))
                         {
-                            TimeStampToken timeStampToken = new TimeStampToken(new CMSSignedData(contents.getBytes()));
-                            System.out.println("Time stamp gen time: " + timeStampToken.getTimeStampInfo().getGenTime());
-                            System.out.println("Time stamp tsa name: " + timeStampToken.getTimeStampInfo().getTsa().getName());
-
-                            CertificateFactory factory = CertificateFactory.getInstance("X.509");
-                            ByteArrayInputStream certStream = new ByteArrayInputStream(contents.getBytes());
-                            Collection<? extends Certificate> certs = factory.generateCertificates(certStream);
-                            System.out.println("certs=" + certs);
-
-                            //TODO verify signature
+                            // e.g. PDFBOX-1848, file_timestamped.pdf
+                            verifyETSIdotRFC3161(buf, contents);
                         }
                         else
                         {
@@ -247,20 +288,55 @@ public final class ShowSignature
         }
     }
 
+    private void verifyETSIdotRFC3161(byte[] buf, COSString contents)
+            throws CertificateException, CMSException, IOException, OperatorCreationException,
+            TSPException, NoSuchAlgorithmException, CertificateVerificationException
+    {
+        TimeStampToken timeStampToken = new TimeStampToken(new CMSSignedData(contents.getBytes()));
+        System.out.println("Time stamp gen time: " + timeStampToken.getTimeStampInfo().getGenTime());
+        System.out.println("Time stamp tsa name: " + timeStampToken.getTimeStampInfo().getTsa().getName());
+        
+        CertificateFactory factory = CertificateFactory.getInstance("X.509");
+        ByteArrayInputStream certStream = new ByteArrayInputStream(contents.getBytes());
+        Collection<? extends Certificate> certs = factory.generateCertificates(certStream);
+        System.out.println("certs=" + certs);
+        
+        String hashAlgorithm = timeStampToken.getTimeStampInfo().getMessageImprintAlgOID().getId();
+        // compare the hash of the signed content with the hash in
+        // the timestamp
+        if (Arrays.equals(MessageDigest.getInstance(hashAlgorithm).digest(buf),
+                timeStampToken.getTimeStampInfo().getMessageImprintDigest()))
+        {
+            System.out.println("ETSI.RFC3161 timestamp signature verified");
+        }
+        else
+        {
+            System.err.println("ETSI.RFC3161 timestamp signature verification failed");
+        }
+
+        X509Certificate certFromTimeStamp = (X509Certificate) certs.iterator().next();
+        SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
+        validateTimestampToken(timeStampToken);
+                verifyCertificateChain(timeStampToken.getCertificates(),
+                certFromTimeStamp,
+                timeStampToken.getTimeStampInfo().getGenTime());
+    }
+
     /**
      * Verify a PKCS7 signature.
      *
      * @param byteArray the byte sequence that has been signed
      * @param contents the /Contents field as a COSString
      * @param sig the PDF signature (the /V dictionary)
-     * @throws CertificateException
      * @throws CMSException
-     * @throws StoreException
      * @throws OperatorCreationException
+     * @throws IOException
+     * @throws GeneralSecurityException
+     * @throws TSPException
      */
     private void verifyPKCS7(byte[] byteArray, COSString contents, PDSignature sig)
-            throws CMSException, CertificateException, StoreException, OperatorCreationException,
-                   NoSuchAlgorithmException, NoSuchProviderException
+            throws CMSException, OperatorCreationException,
+                   IOException, GeneralSecurityException, TSPException, CertificateVerificationException
     {
         // inspiration:
         // http://stackoverflow.com/a/26702631/535646
@@ -277,19 +353,75 @@ public final class ShowSignature
         X509CertificateHolder certificateHolder = matches.iterator().next();
         X509Certificate certFromSignedData = new JcaX509CertificateConverter().getCertificate(certificateHolder);
         System.out.println("certFromSignedData: " + certFromSignedData);
-        certFromSignedData.checkValidity(sig.getSignDate().getTime());
 
-        if (isSelfSigned(certFromSignedData))
+        SigUtils.checkCertificateUsage(certFromSignedData);
+
+        // Embedded timestamp
+        TimeStampToken timeStampToken = extractTimeStampTokenFromSignerInformation(signerInformation);
+        if (timeStampToken != null)
         {
-            System.err.println("Certificate is self-signed, LOL!");
-        }
-        else
-        {
-            System.out.println("Certificate is not self-signed");
-            // todo rest of chain
+            // tested with QV_RCA1_RCA3_CPCPS_V4_11.pdf
+            // https://www.quovadisglobal.com/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_11.ashx
+            // timeStampToken.getCertificates() only contained the local certificate and not
+            // the whole chain, so use the store of the main signature.
+            // (If this assumption is incorrect, then the code must be changed to merge
+            // both stores, or to pass a collection)
+            validateTimestampToken(timeStampToken);
+            X509CertificateHolder tstCertHolder = (X509CertificateHolder) timeStampToken.getCertificates().getMatches(null).iterator().next();
+            X509Certificate certFromTimeStamp = new JcaX509CertificateConverter().getCertificate(tstCertHolder);
+            verifyCertificateChain(certificatesStore,
+                    certFromTimeStamp,
+                    timeStampToken.getTimeStampInfo().getGenTime());
+            SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
         }
 
-        if (signerInformation.verify(new JcaSimpleSignerInfoVerifierBuilder().build(certFromSignedData)))
+        try
+        {
+            if (sig.getSignDate() != null)
+            {
+                certFromSignedData.checkValidity(sig.getSignDate().getTime());
+                System.out.println("Certificate valid at signing time");
+            }
+            else
+            {
+                System.err.println("Certificate cannot be verified without signing time");
+            }
+        }
+        catch (CertificateExpiredException ex)
+        {
+            System.err.println("Certificate expired at signing time");
+        }
+        catch (CertificateNotYetValidException ex)
+        {
+            System.err.println("Certificate not yet valid at signing time");
+        }
+
+        // usually not available
+        if (signerInformation.getSignedAttributes() != null)
+        {
+            // From SignedMailValidator.getSignatureTime()
+            Attribute signingTime = signerInformation.getSignedAttributes().get(CMSAttributes.signingTime);
+            if (signingTime != null)
+            {
+                Time timeInstance = Time.getInstance(signingTime.getAttrValues().getObjectAt(0));
+                try
+                {
+                    certFromSignedData.checkValidity(timeInstance.getDate());
+                    System.out.println("Certificate valid at signing time: " + timeInstance.getDate());
+                }
+                catch (CertificateExpiredException ex)
+                {
+                    System.err.println("Certificate expired at signing time");
+                }
+                catch (CertificateNotYetValidException ex)
+                {
+                    System.err.println("Certificate not yet valid at signing time");
+                }
+            }
+        }
+
+        if (signerInformation.verify(new JcaSimpleSignerInfoVerifierBuilder().
+                setProvider(SecurityProvider.getProvider()).build(certFromSignedData)))
         {
             System.out.println("Signature verified");
         }
@@ -297,6 +429,75 @@ public final class ShowSignature
         {
             System.out.println("Signature verification failed");
         }
+
+        if (CertificateVerifier.isSelfSigned(certFromSignedData))
+        {
+            System.err.println("Certificate is self-signed, LOL!");
+        }
+        else
+        {
+            System.out.println("Certificate is not self-signed");
+
+            if (sig.getSignDate() != null)
+            {
+                verifyCertificateChain(certificatesStore, certFromSignedData, sig.getSignDate().getTime());
+            }
+            else
+            {
+                System.err.println("Certificate cannot be verified without signing time");
+            }
+        }
+    }
+
+    private TimeStampToken extractTimeStampTokenFromSignerInformation(SignerInformation signerInformation)
+            throws CMSException, IOException, TSPException
+    {
+        if (signerInformation.getUnsignedAttributes() == null)
+        {
+            return null;
+        }
+        AttributeTable unsignedAttributes = signerInformation.getUnsignedAttributes();
+        // https://stackoverflow.com/questions/1647759/how-to-validate-if-a-signed-jar-contains-a-timestamp
+        Attribute attribute = unsignedAttributes.get(
+                PKCSObjectIdentifiers.id_aa_signatureTimeStampToken);
+        ASN1Object obj = (ASN1Object) attribute.getAttrValues().getObjectAt(0);
+        CMSSignedData signedTSTData = new CMSSignedData(obj.getEncoded());
+        return new TimeStampToken(signedTSTData);
+    }
+
+    private void verifyCertificateChain(Store<X509CertificateHolder> certificatesStore,
+            X509Certificate certFromSignedData, Date signDate)
+            throws CertificateVerificationException, CertificateException
+    {
+        // Verify certificate chain (new since 11/2018)
+        // Please post bad PDF files that succeed and
+        // good PDF files that fail in
+        // https://issues.apache.org/jira/browse/PDFBOX-3017
+        Collection<X509CertificateHolder> certificateHolders = certificatesStore.getMatches(null);
+        Set<X509Certificate> additionalCerts = new HashSet<X509Certificate>();
+        JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
+        for (X509CertificateHolder certHolder : certificateHolders)
+        {
+            X509Certificate certificate = certificateConverter.getCertificate(certHolder);
+            if (!certificate.equals(certFromSignedData))
+            {
+                additionalCerts.add(certificate);
+            }
+        }
+        CertificateVerifier.verifyCertificate(certFromSignedData, additionalCerts, true, signDate);
+    }
+
+    private void validateTimestampToken(TimeStampToken timeStampToken)
+            throws IOException, CertificateException, TSPException, OperatorCreationException
+    {
+        // https://stackoverflow.com/questions/42114742/
+        Collection<X509CertificateHolder> tstMatches =
+                timeStampToken.getCertificates().getMatches(timeStampToken.getSID());
+        X509CertificateHolder holder = tstMatches.iterator().next();
+        X509Certificate tstCert = new JcaX509CertificateConverter().getCertificate(holder);
+        SignerInformationVerifier siv = new JcaSimpleSignerInfoVerifierBuilder().setProvider(SecurityProvider.getProvider()).build(tstCert);
+        timeStampToken.validate(siv);
+        System.out.println("TimeStampToken validated");
     }
 
     /**
@@ -358,31 +559,6 @@ public final class ShowSignature
         }
     }
 
-    // https://svn.apache.org/repos/asf/cxf/tags/cxf-2.4.1/distribution/src/main/release/samples/sts_issue_operation/src/main/java/demo/sts/provider/cert/CertificateVerifier.java
-    
-    /**
-     * Checks whether given X.509 certificate is self-signed.
-     */
-    private boolean isSelfSigned(X509Certificate cert)
-            throws CertificateException, NoSuchAlgorithmException, NoSuchProviderException
-    {
-        try
-        {
-            // Try to verify certificate signature with its own public key
-            PublicKey key = cert.getPublicKey();
-            cert.verify(key);
-            return true;
-        }
-        catch (SignatureException sigEx)
-        {
-            return false;
-        }
-        catch (InvalidKeyException keyEx)
-        {
-            return false;
-        }
-    }
-    
     /**
      * This will print a usage message.
      */
