@@ -24,9 +24,11 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.math.BigInteger;
 import java.security.GeneralSecurityException;
-import java.security.cert.CRLException;
+import java.security.Security;
 import java.security.cert.CertificateEncodingException;
+import java.security.cert.X509CRL;
 import java.security.cert.X509Certificate;
+import java.util.Calendar;
 import java.util.HashSet;
 import java.util.Set;
 
@@ -35,13 +37,20 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
-import org.apache.pdfbox.cos.COSInteger;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.cos.COSUpdateInfo;
+import org.apache.pdfbox.examples.signature.SigUtils;
+import org.apache.pdfbox.examples.signature.cert.CRLVerifier;
+import org.apache.pdfbox.examples.signature.cert.CertificateVerificationException;
+import org.apache.pdfbox.examples.signature.cert.OcspHelper;
+import org.apache.pdfbox.examples.signature.cert.RevokedCertificateException;
 import org.apache.pdfbox.examples.signature.validation.CertInformationCollector.CertSignatureInformation;
+import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
+import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
+import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
 import org.bouncycastle.cert.ocsp.BasicOCSPResp;
 import org.bouncycastle.cert.ocsp.OCSPException;
 import org.bouncycastle.cert.ocsp.OCSPResp;
@@ -67,6 +76,7 @@ public class AddValidationInformation
     private COSArray certs;
     private PDDocument document;
     private final Set<BigInteger> foundRevocationInformation = new HashSet<BigInteger>();
+    private Calendar signDate;
 
     /**
      * Signs the given PDF file.
@@ -91,9 +101,9 @@ public class AddValidationInformation
     }
 
     /**
-     * Fetches certificate information from the last signature of the document and appends a DSS with the validation
-     * information to the document.
-     * 
+     * Fetches certificate information from the last signature of the document and appends a DSS
+     * with the validation information to the document.
+     *
      * @param document containing the Signature
      * @param filename in file to extract signature
      * @param output where to write the changed document
@@ -102,10 +112,15 @@ public class AddValidationInformation
     private void doValidation(String filename, OutputStream output) throws IOException
     {
         certInformationHelper = new CertInformationCollector();
-        CertSignatureInformation certInfo;
+        CertSignatureInformation certInfo = null;
         try
         {
-            certInfo = certInformationHelper.getLastCertInfo(document, filename);
+            PDSignature signature = SigUtils.getLastRelevantSignature(document);
+            if (signature != null)
+            {
+                certInfo = certInformationHelper.getLastCertInfo(signature, filename);
+                signDate = signature.getSignDate();
+            }
         }
         catch (CertificateProccessingException e)
         {
@@ -190,14 +205,15 @@ public class AddValidationInformation
 
     /**
      * Fetches and adds revocation information based on the certInfo to the DSS.
-     * 
-     * @param certInfo Certificate information from CertInformationHelper containing certificate chains.
+     *
+     * @param certInfo Certificate information from CertInformationHelper containing certificate
+     * chains.
      * @throws IOException
      */
     private void addRevocationData(CertSignatureInformation certInfo) throws IOException
     {
         COSDictionary vri = new COSDictionary();
-        vriBase.setItem(COSName.getPDFName(certInfo.getSignatureHash()), vri);
+        vriBase.setItem(certInfo.getSignatureHash(), vri);
 
         correspondingOCSPs = new COSArray();
         correspondingCRLs = new COSArray();
@@ -206,11 +222,11 @@ public class AddValidationInformation
 
         if (correspondingOCSPs.size() > 0)
         {
-            vri.setItem(COSName.getPDFName("OCSP"), correspondingOCSPs);
+            vri.setItem("OCSP", correspondingOCSPs);
         }
         if (correspondingCRLs.size() > 0)
         {
-            vri.setItem(COSName.getPDFName("CRL"), correspondingCRLs);
+            vri.setItem("CRL", correspondingCRLs);
         }
 
         if (certInfo.getTsaCerts() != null)
@@ -224,8 +240,9 @@ public class AddValidationInformation
 
     /**
      * Tries to get Revocation Data (first OCSP, else CRL) from the given Certificate Chain.
-     * 
-     * @param certInfo from which to fetch revocation data. Will work recursively through its chains.
+     *
+     * @param certInfo from which to fetch revocation data. Will work recursively through its
+     * chains.
      * @throws IOException when failed to fetch an revocation data.
      */
     private void addRevocationDataRecursive(CertSignatureInformation certInfo) throws IOException
@@ -249,10 +266,14 @@ public class AddValidationInformation
                 isRevocationInfoFound = true;
             }
 
-            if (!isRevocationInfoFound)
+            if (certInfo.getOcspUrl() == null && certInfo.getCrlUrl() == null)
+            {
+                LOG.info("No revocation information for cert " + certInfo.getCertificate().getSubjectX500Principal());
+            }
+            else if (!isRevocationInfoFound)
             {
                 throw new IOException("Could not fetch Revocation Info for Cert: "
-                        + certInfo.getCertificate().getSubjectDN());
+                        + certInfo.getCertificate().getSubjectX500Principal());
             }
         }
 
@@ -269,7 +290,7 @@ public class AddValidationInformation
 
     /**
      * Tries to fetch and add OCSP Data to its containers.
-     * 
+     *
      * @param certInfo the certificate info, for it to check OCSP data.
      * @return true when the OCSP data has successfully been fetched and added
      * @throws IOException when Certificate is revoked.
@@ -304,9 +325,10 @@ public class AddValidationInformation
 
     /**
      * Tries to fetch and add CRL Data to its containers.
-     * 
+     *
      * @param certInfo the certificate info, for it to check CRL data.
-     * @throws IOException when failed to fetch, because no validation data could be fetched for data.
+     * @throws IOException when failed to fetch, because no validation data could be fetched for
+     * data.
      */
     private void fetchCrlData(CertSignatureInformation certInfo) throws IOException
     {
@@ -314,7 +336,7 @@ public class AddValidationInformation
         {
             addCrlRevocationInfo(certInfo);
         }
-        catch (CRLException e)
+        catch (GeneralSecurityException e)
         {
             LOG.warn("Failed fetching CRL", e);
             throw new IOException(e);
@@ -329,6 +351,11 @@ public class AddValidationInformation
             LOG.warn("Failed fetching CRL", e);
             throw new IOException(e);
         }
+        catch (CertificateVerificationException e)
+        {
+            LOG.warn("Failed fetching CRL", e);
+            throw new IOException(e);
+        }
     }
 
     /**
@@ -343,12 +370,19 @@ public class AddValidationInformation
     private void addOcspData(CertSignatureInformation certInfo) throws IOException, OCSPException,
             CertificateProccessingException, RevokedCertificateException
     {
-        OcspHelper ocspHelper = new OcspHelper(certInfo.getCertificate(),
-                certInfo.getIssuerCertificate(), certInfo.getOcspUrl());
+        OcspHelper ocspHelper = new OcspHelper(
+                certInfo.getCertificate(),
+                signDate.getTime(),
+                certInfo.getIssuerCertificate(),
+                new HashSet<X509Certificate>(certInformationHelper.getCertificatesMap().values()),
+                certInfo.getOcspUrl());
 
         OCSPResp ocspResp = ocspHelper.getResponseOcsp();
         BasicOCSPResp basicResponse = (BasicOCSPResp) ocspResp.getResponseObject();
         certInformationHelper.addAllCertsFromHolders(basicResponse.getCerts());
+        
+        //TODO check revocation of responder + include in separate VRI (usually not needed). 
+        // See comment by mkl in PDFBOX-3017 on 21.11.2018
 
         byte[] ocspData = ocspResp.getEncoded();
 
@@ -365,16 +399,19 @@ public class AddValidationInformation
      * Fetches and adds CRL data to storage for the given Certificate.
      * 
      * @param certInfo the certificate info, for it to check CRL data.
-     * @throws CRLException
      * @throws IOException
      * @throws RevokedCertificateException
+     * @throws GeneralSecurityException
+     * @throws CertificateVerificationException 
      */
     private void addCrlRevocationInfo(CertSignatureInformation certInfo)
-            throws CRLException, IOException, RevokedCertificateException
+            throws IOException, RevokedCertificateException, GeneralSecurityException,
+            CertificateVerificationException
     {
-        byte[] crlData = CrlHelper.performCrlRequestAndCheck(certInfo.getCrlUrl(),
-                certInfo.getCertificate());
-        COSStream crlStream = writeDataToStream(crlData);
+        X509CRL crl = CRLVerifier.downloadCRLFromWeb(certInfo.getCrlUrl());
+        crl.verify(certInfo.getIssuerCertificate().getPublicKey(), SecurityProvider.getProvider().getName());
+        CRLVerifier.checkRevocation(crl, certInfo.getCertificate(), signDate.getTime(), certInfo.getCrlUrl());
+        COSStream crlStream = writeDataToStream(crl.getEncoded());
         crls.add(crlStream);
         if (correspondingCRLs != null)
         {
@@ -384,16 +421,16 @@ public class AddValidationInformation
     }
 
     /**
-     * Adds all certs to the certs-array. Make sure, all certificates are inside the certificateStore of
-     * certInformationHelper
-     * 
+     * Adds all certs to the certs-array. Make sure, all certificates are inside the
+     * certificateStore of certInformationHelper
+     *
      * @throws IOException
      */
     private void addAllCertsToCertArray() throws IOException
     {
         try
         {
-            for (X509Certificate cert : certInformationHelper.getCertificateStore().values())
+            for (X509Certificate cert : certInformationHelper.getCertificatesMap().values())
             {
                 COSStream stream = writeDataToStream(cert.getEncoded());
                 certs.add(stream);
@@ -406,45 +443,48 @@ public class AddValidationInformation
     }
 
     /**
-     * Creates a FlateDecoded <code>COSStream</code> element with the given data.
+     * Creates a Flate encoded <code>COSStream</code> object with the given data.
      * 
-     * @param data to write into the element
-     * @return COSStream Element, that can be added to the document
+     * @param data to write into the COSStream
+     * @return COSStream a COSStream object that can be added to the document
      * @throws IOException
      */
     private COSStream writeDataToStream(byte[] data) throws IOException
     {
         COSStream stream = document.getDocument().createCOSStream();
-        COSArray filters = new COSArray();
-        filters.add(COSName.FLATE_DECODE);
-
-        OutputStream os = stream.createOutputStream(filters);
-        os.write(data);
-        os.close();
-
+        OutputStream os = null;
+        try
+        {
+            os = stream.createOutputStream(COSName.FLATE_DECODE);
+            os.write(data);
+        }
+        finally
+        {
+            IOUtils.closeQuietly(os);
+        }
         return stream;
     }
 
     /**
-     * Adds Extensions to the document catalog. So that the use of DSS is identified. Described in PAdES Part 4, Chapter
-     * 4.4.
-     * 
+     * Adds Extensions to the document catalog. So that the use of DSS is identified. Described in
+     * PAdES Part 4, Chapter 4.4.
+     *
      * @param catalog to add Extensions into
      */
     private void addExtensions(PDDocumentCatalog catalog)
     {
         COSDictionary dssExtensions = new COSDictionary();
         dssExtensions.setDirect(true);
-        catalog.getCOSObject().setItem(COSName.getPDFName("Extensions"), dssExtensions);
+        catalog.getCOSObject().setItem("Extensions", dssExtensions);
 
         COSDictionary adbeExtension = new COSDictionary();
         adbeExtension.setDirect(true);
-        dssExtensions.setItem(COSName.getPDFName("ADBE"), adbeExtension);
+        dssExtensions.setItem("ADBE", adbeExtension);
 
-        adbeExtension.setItem(COSName.getPDFName("BaseVersion"), COSName.getPDFName("1.7"));
-        adbeExtension.setItem(COSName.getPDFName("ExtensionLevel"), COSInteger.get(5));
+        adbeExtension.setName("BaseVersion", "1.7");
+        adbeExtension.setInt("ExtensionLevel", 5);
 
-        catalog.getCOSObject().setItem(COSName.getPDFName("Version"), COSName.getPDFName("1.7"));
+        catalog.setVersion("1.7");
     }
 
     public static void main(String[] args) throws IOException, GeneralSecurityException
@@ -455,6 +495,9 @@ public class AddValidationInformation
             System.exit(1);
         }
 
+        // register BouncyCastle provider, needed for "exotic" algorithms
+        Security.addProvider(SecurityProvider.getProvider());
+
         // add ocspInformation
         AddValidationInformation addOcspInformation = new AddValidationInformation();
 
