@@ -22,27 +22,31 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.math.BigInteger;
 import java.net.URL;
+import java.security.GeneralSecurityException;
+import java.security.InvalidKeyException;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.SignatureException;
 import java.security.cert.CertificateException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.SortedMap;
-import java.util.TreeMap;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.apache.pdfbox.cos.COSBase;
-import org.apache.pdfbox.cos.COSName;
+import org.apache.pdfbox.examples.signature.cert.CertificateVerifier;
 import org.apache.pdfbox.io.IOUtils;
-import org.apache.pdfbox.pdmodel.PDDocument;
+import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
 import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.cms.Attribute;
 import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.x509.Extension;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
 import org.bouncycastle.cms.CMSException;
@@ -50,6 +54,7 @@ import org.bouncycastle.cms.CMSSignedData;
 import org.bouncycastle.cms.SignerInformation;
 import org.bouncycastle.tsp.TSPException;
 import org.bouncycastle.tsp.TimeStampToken;
+import org.bouncycastle.util.Selector;
 import org.bouncycastle.util.Store;
 
 /**
@@ -64,78 +69,37 @@ public class CertInformationCollector
 {
     private static final Log LOG = LogFactory.getLog(CertInformationCollector.class);
 
-    // As described in https://tools.ietf.org/html/rfc3280.html#section-4.2.2.1
-    private static final String ID_PE_AUTHORITYINFOACCESS = "1.3.6.1.5.5.7.1.1";
-
-    // As described in https://tools.ietf.org/html/rfc3280.html#section-4.2.1.14
-    // Disable false Sonar warning for "Hardcoded IP Address ..."
-    @SuppressWarnings("squid:S1313")
-    private static final String ID_CE_CRLDISTRIBUTIONPOINTS = "2.5.29.31";
-
     private static final int MAX_CERTIFICATE_CHAIN_DEPTH = 5;
 
-    private final Map<BigInteger, X509Certificate> certificateStore = new HashMap<BigInteger, X509Certificate>();
+    private final Map<BigInteger, X509Certificate> certificatesMap = new HashMap<BigInteger, X509Certificate>();
 
     private final JcaX509CertificateConverter certConverter = new JcaX509CertificateConverter();
 
     private CertSignatureInformation rootCertInfo;
 
     /**
-     * Gets the Certificate Information of the last Signature.
+     * Gets the certificate information of a signature.
      * 
-     * @param document to get the Signature from
+     * @param signature the signature of the document.
      * @param fileName of the document.
      * @return the CertSignatureInformation containing all certificate information
      * @throws CertificateProccessingException when there is an error processing the certificates
      * @throws IOException on a data processing error
      */
-    public CertSignatureInformation getLastCertInfo(PDDocument document, String fileName)
+    public CertSignatureInformation getLastCertInfo(PDSignature signature, String fileName)
             throws CertificateProccessingException, IOException
     {
-        PDSignature signature = getLastRelevantSignature(document);
-        if (signature != null)
+        FileInputStream documentInput = null;
+        try
         {
-            FileInputStream documentInput = null;
-            try
-            {
-                documentInput = new FileInputStream(fileName);
-                byte[] docBytes = IOUtils.toByteArray(documentInput);
-                byte[] signatureContent = signature.getContents(docBytes);
-                return getCertInfo(signatureContent);
-            }
-            finally
-            {
-                IOUtils.closeQuietly(document);
-            }
+            documentInput = new FileInputStream(fileName);
+            byte[] signatureContent = signature.getContents(documentInput);
+            return getCertInfo(signatureContent);
         }
-        return null;
-    }
-
-    /**
-     * Gets the last relevant signature in the document
-     * 
-     * @param document to get its last Signature
-     * @return last signature or null when none found
-     * @throws IOException
-     */
-    private PDSignature getLastRelevantSignature(PDDocument document) throws IOException
-    {
-        SortedMap<Integer, PDSignature> sortedMap = new TreeMap<Integer, PDSignature>();
-        for (PDSignature signature : document.getSignatureDictionaries())
+        finally
         {
-            int sigOffset = signature.getByteRange()[1];
-            sortedMap.put(sigOffset, signature);
+            IOUtils.closeQuietly(documentInput);
         }
-        if (sortedMap.size() > 0)
-        {
-            PDSignature lastSignature = sortedMap.get(sortedMap.lastKey());
-            COSBase type = lastSignature.getCOSObject().getItem(COSName.TYPE);
-            if (type.equals(COSName.SIG) || type.equals(COSName.DOC_TIME_STAMP))
-            {
-                return lastSignature;
-            }
-        }
-        return null;
     }
 
     /**
@@ -225,10 +189,10 @@ public class CertInformationCollector
      * not yet practicable.
      *
      * @param certificatesStore To get the certificate information from. Certificates will be saved
-     * in the certificateStore.
-     * @param signedData to get SignerInformation off
+     * in certificatesMap.
+     * @param signedData data from which to get the SignerInformation
      * @param certInfo where to add certificate information
-     * @return Signer Information of the processed certificate Store for further usage.
+     * @return Signer Information of the processed certificatesStore for further usage.
      * @throws IOException on data-processing error
      * @throws CertificateProccessingException on a specific error with a certificate
      */
@@ -239,8 +203,9 @@ public class CertInformationCollector
         Collection<SignerInformation> signers = signedData.getSignerInfos().getSigners();
         SignerInformation signerInformation = signers.iterator().next();
 
+        @SuppressWarnings("unchecked")
         Collection<X509CertificateHolder> matches = certificatesStore
-                .getMatches(signerInformation.getSID());
+                .getMatches((Selector<X509CertificateHolder>) signerInformation.getSID());
 
         X509Certificate certificate = getCertFromHolder(matches.iterator().next());
 
@@ -266,7 +231,8 @@ public class CertInformationCollector
         certInfo.certificate = certificate;
 
         // Certificate Authority Information Access
-        byte[] authorityExtensionValue = certificate.getExtensionValue(ID_PE_AUTHORITYINFOACCESS);
+        // As described in https://tools.ietf.org/html/rfc3280.html#section-4.2.2.1
+        byte[] authorityExtensionValue = certificate.getExtensionValue(Extension.authorityInfoAccess.getId());
         if (authorityExtensionValue != null)
         {
             CertInformationHelper.getAuthorityInfoExtensionValue(authorityExtensionValue, certInfo);
@@ -277,37 +243,66 @@ public class CertInformationCollector
             getAlternativeIssuerCertificate(certInfo, maxDepth);
         }
 
-        byte[] crlExtensionValue = certificate.getExtensionValue(ID_CE_CRLDISTRIBUTIONPOINTS);
+        // As described in https://tools.ietf.org/html/rfc3280.html#section-4.2.1.14
+        byte[] crlExtensionValue = certificate.getExtensionValue(Extension.cRLDistributionPoints.getId());
         if (crlExtensionValue != null)
         {
             certInfo.crlUrl = CertInformationHelper.getCrlUrlFromExtensionValue(crlExtensionValue);
         }
 
-        if (CertInformationHelper.isSelfSigned(certificate))
+        try
         {
-            certInfo.isSelfSigned = true;
+            certInfo.isSelfSigned = CertificateVerifier.isSelfSigned(certificate);
+        }
+        catch (GeneralSecurityException ex)
+        {
+            throw new CertificateProccessingException(ex);
         }
         if (maxDepth <= 0 || certInfo.isSelfSigned)
         {
             return;
         }
 
-        for (X509Certificate issuer : certificateStore.values())
+        for (X509Certificate issuer : certificatesMap.values())
         {
-            if (CertInformationHelper.verify(certificate, issuer.getPublicKey()))
+            if (certificate.getIssuerX500Principal().equals(issuer.getSubjectX500Principal()))
             {
-                LOG.info("Found the right Issuer Cert! for Cert: " + certificate.getSubjectDN()
-                        + "\n" + issuer.getSubjectDN());
+                try
+                {
+                    certificate.verify(issuer.getPublicKey(), SecurityProvider.getProvider().getName());
+                }
+                catch (CertificateException ex)
+                {
+                    throw new CertificateProccessingException(ex);
+                }
+                catch (NoSuchAlgorithmException ex)
+                {
+                    throw new CertificateProccessingException(ex);
+                }
+                catch (InvalidKeyException ex)
+                {
+                    throw new CertificateProccessingException(ex);
+                }
+                catch (SignatureException ex)
+                {
+                    throw new CertificateProccessingException(ex);
+                }
+                catch (NoSuchProviderException ex)
+                {
+                    throw new CertificateProccessingException(ex);
+                }
+                LOG.info("Found the right Issuer Cert! for Cert: " + certificate.getSubjectX500Principal()
+                        + "\n" + issuer.getSubjectX500Principal());
                 certInfo.issuerCertificate = issuer;
                 certInfo.certChain = new CertSignatureInformation();
-                traverseChain(issuer, certInfo.certChain, --maxDepth);
+                traverseChain(issuer, certInfo.certChain, maxDepth - 1);
                 break;
             }
         }
         if (certInfo.issuerCertificate == null)
         {
             throw new IOException(
-                    "No Issuer Certificate found for Cert: " + certificate.getSubjectDN());
+                    "No Issuer Certificate found for Cert: " + certificate.getSubjectX500Principal());
         }
     }
 
@@ -324,7 +319,7 @@ public class CertInformationCollector
     private void getAlternativeIssuerCertificate(CertSignatureInformation certInfo, int maxDepth)
             throws CertificateProccessingException
     {
-        System.out.println("Get Certificate from: " + certInfo.issuerUrl);
+        LOG.info("Get alternative issuer certificate from: " + certInfo.issuerUrl);
         try
         {
             URL certUrl = new URL(certInfo.issuerUrl);
@@ -332,7 +327,7 @@ public class CertInformationCollector
             InputStream in = certUrl.openStream();
 
             X509Certificate altIssuerCert = (X509Certificate) certFactory.generateCertificate(in);
-            addCertToCertStore(altIssuerCert);
+            addCertToCertificatesMap(altIssuerCert);
 
             certInfo.alternativeCertChain = new CertSignatureInformation();
             traverseChain(altIssuerCert, certInfo.alternativeCertChain, maxDepth - 1);
@@ -340,30 +335,30 @@ public class CertInformationCollector
         }
         catch (IOException e)
         {
-            LOG.error("Error getting additional Certificate from " + certInfo.issuerUrl, e);
+            LOG.error("Error getting alternative issuer certificate from " + certInfo.issuerUrl, e);
         }
         catch (CertificateException e)
         {
-            LOG.error("Error getting additional Certificate from " + certInfo.issuerUrl, e);
+            LOG.error("Error getting alternative issuer certificate from " + certInfo.issuerUrl, e);
         }
     }
 
     /**
-     * Adds the given Certificate to the certificateStore, if not yet containing.
+     * Adds the given Certificate to the certificatesMap, if not yet containing.
      * 
-     * @param certificate to add to the certificateStore
+     * @param certificate to add to the certificatesMap
      */
-    private void addCertToCertStore(X509Certificate certificate)
+    private void addCertToCertificatesMap(X509Certificate certificate)
     {
-        if (!certificateStore.containsKey(certificate.getSerialNumber()))
+        if (!certificatesMap.containsKey(certificate.getSerialNumber()))
         {
-            certificateStore.put(certificate.getSerialNumber(), certificate);
+            certificatesMap.put(certificate.getSerialNumber(), certificate);
         }
     }
 
     /**
-     * Gets the X509Certificate out of the X509CertificateHolder
-     * 
+     * Gets the X509Certificate out of the X509CertificateHolder and add it to certificatesMap.
+     *
      * @param certificateHolder to get the certificate from
      * @return a X509Certificate or <code>null</code> when there was an Error with the Certificate
      * @throws CertificateProccessingException on failed conversion from X509CertificateHolder to X509Certificate
@@ -371,12 +366,13 @@ public class CertInformationCollector
     private X509Certificate getCertFromHolder(X509CertificateHolder certificateHolder)
             throws CertificateProccessingException
     {
-        if (!certificateStore.containsKey(certificateHolder.getSerialNumber()))
+        //TODO getCertFromHolder violates "do one thing" rule (adds to the map and returns a certificate)
+        if (!certificatesMap.containsKey(certificateHolder.getSerialNumber()))
         {
             try
             {
                 X509Certificate certificate = certConverter.getCertificate(certificateHolder);
-                certificateStore.put(certificate.getSerialNumber(), certificate);
+                certificatesMap.put(certificate.getSerialNumber(), certificate);
                 return certificate;
             }
             catch (CertificateException e)
@@ -387,13 +383,12 @@ public class CertInformationCollector
         }
         else
         {
-            return certificateStore.get(certificateHolder.getSerialNumber());
+            return certificatesMap.get(certificateHolder.getSerialNumber());
         }
     }
 
     /**
-     * Adds multiple Certificates out of a Collection of X509CertificateHolder into the
-     * Certificate-Store.
+     * Adds multiple Certificates out of a Collection of X509CertificateHolder into certificatesMap.
      *
      * @param certHolders Collection of X509CertificateHolder
      */
@@ -414,7 +409,7 @@ public class CertInformationCollector
 
     /**
      * Gets a list of X509Certificate out of an array of X509CertificateHolder. The certificates
-     * will be added to the certificateStore.
+     * will be added to certificatesMap.
      *
      * @param certHolders Array of X509CertificateHolder
      * @throws CertificateProccessingException when one of the Certificates could not be parsed.
@@ -422,20 +417,42 @@ public class CertInformationCollector
     public void addAllCertsFromHolders(X509CertificateHolder[] certHolders)
             throws CertificateProccessingException
     {
-        for (X509CertificateHolder certHolder : certHolders)
+        addAllCerts(Arrays.asList(certHolders));
+    }
+
+    /**
+     * Traverse the OCSP certificate.
+     *
+     * @param certHolder
+     * @return
+     * @throws CertificateProccessingException 
+     */
+    CertSignatureInformation getOCSPCertInfo(X509CertificateHolder certHolder) throws CertificateProccessingException
+    {
+        try
         {
-            getCertFromHolder(certHolder);
+            CertSignatureInformation certSignatureInformation = new CertSignatureInformation();
+            traverseChain(certConverter.getCertificate(certHolder), certSignatureInformation, MAX_CERTIFICATE_CHAIN_DEPTH);
+            return certSignatureInformation;
+        }
+        catch (CertificateException ex)
+        {
+            throw new CertificateProccessingException(ex);
+        }
+        catch (IOException ex)
+        {
+            throw new CertificateProccessingException(ex);
         }
     }
 
     /**
-     * Get the certificate store of all processed certificates until now.
+     * Get the map of all processed certificates until now.
      * 
      * @return a map of serial numbers to certificates.
      */
-    public Map<BigInteger, X509Certificate> getCertificateStore()
+    public Map<BigInteger, X509Certificate> getCertificatesMap()
     {
-        return certificateStore;
+        return certificatesMap;
     }
 
     /**
