@@ -24,6 +24,8 @@ import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 import org.apache.fontbox.util.Charsets;
 
@@ -33,6 +35,11 @@ import org.apache.fontbox.util.Charsets;
  */
 public class CFFParser
 {
+    /**
+     * Log instance.
+     */
+    private static final Log LOG = LogFactory.getLog(CFFParser.class);
+
     private static final String TAG_OTTO = "OTTO";
     private static final String TAG_TTCF = "ttcf";
     private static final String TAG_TTFONLY = "\u0000\u0001\u0000\u0000";
@@ -83,37 +90,7 @@ public class CFFParser
         // try to determine which kind of font we have
         if (TAG_OTTO.equals(firstTag))
         {
-            // this is OpenType font containing CFF data
-            // so find CFF tag
-            short numTables = input.readShort();
-            @SuppressWarnings("unused")
-            short searchRange = input.readShort();
-            @SuppressWarnings("unused")
-            short entrySelector = input.readShort();
-            @SuppressWarnings("unused")
-            short rangeShift = input.readShort();
-
-            boolean cffFound = false;
-            for (int q = 0; q < numTables; q++)
-            {
-                String tagName = readTagName(input);
-                @SuppressWarnings("unused")
-                long checksum = readLong(input);
-                long offset = readLong(input);
-                long length = readLong(input);
-                if (tagName.equals("CFF "))
-                {
-                    cffFound = true;
-                    byte[] bytes2 = new byte[(int) length];
-                    System.arraycopy(bytes, (int) offset, bytes2, 0, bytes2.length);
-                    input = new CFFDataInput(bytes2);
-                    break;
-                }
-            }
-            if (!cffFound)
-            {
-                throw new IOException("CFF tag not found in this OpenType font.");
-            }
+            input = createTaggedCFFDataInput(input, bytes);
         }
         else if (TAG_TTCF.equals(firstTag))
         {
@@ -150,6 +127,33 @@ public class CFFParser
         return fonts;
     }
 
+    private CFFDataInput createTaggedCFFDataInput(CFFDataInput input, byte[] bytes) throws IOException
+    {
+        // this is OpenType font containing CFF data
+        // so find CFF tag
+        short numTables = input.readShort();
+        @SuppressWarnings("unused")
+        short searchRange = input.readShort();
+        @SuppressWarnings("unused")
+        short entrySelector = input.readShort();
+        @SuppressWarnings("unused")
+        short rangeShift = input.readShort();
+        for (int q = 0; q < numTables; q++)
+        {
+            String tagName = readTagName(input);
+            @SuppressWarnings("unused")
+            long checksum = readLong(input);
+            long offset = readLong(input);
+            long length = readLong(input);
+            if ("CFF ".equals(tagName))
+            {
+                byte[] bytes2 = Arrays.copyOfRange(bytes, (int) offset, (int) (offset + length));
+                return new CFFDataInput(bytes2);
+            }
+        }
+        throw new IOException("CFF tag not found in this OpenType font.");
+    }
+
     private static String readTagName(CFFDataInput input) throws IOException
     {
         byte[] b = input.readBytes(4);
@@ -339,6 +343,7 @@ public class CFFParser
         StringBuilder sb = new StringBuilder();
         boolean done = false;
         boolean exponentMissing = false;
+        boolean hasExponent = false;
         while (!done)
         {
             int b = input.readUnsignedByte();
@@ -364,12 +369,24 @@ public class CFFParser
                     sb.append(".");
                     break;
                 case 0xb:
+                    if (hasExponent)
+                    {
+                        LOG.warn("duplicate 'E' ignored after " + sb);
+                        break;
+                    }
                     sb.append("E");
                     exponentMissing = true;
+                    hasExponent = true;
                     break;
                 case 0xc:
+                    if (hasExponent)
+                    {
+                        LOG.warn("duplicate 'E-' ignored after " + sb);
+                        break;
+                    }
                     sb.append("E-");
                     exponentMissing = true;
+                    hasExponent = true;
                     break;
                 case 0xd:
                     break;
@@ -507,7 +524,7 @@ public class CFFParser
 
             List<Number> privMatrix = null;
             List<Map<String, Object>> fontDicts = ((CFFCIDFont) font).getFontDicts();
-            if (fontDicts.size() > 0 && fontDicts.get(0).containsKey("FontMatrix"))
+            if (!fontDicts.isEmpty() && fontDicts.get(0).containsKey("FontMatrix"))
             {
                 privMatrix = (List<Number>) fontDicts.get(0).get("FontMatrix");
             }
@@ -651,17 +668,17 @@ public class CFFParser
     private Map<String, Object> readPrivateDict(DictData privateDict)
     {
         Map<String, Object> privDict = new LinkedHashMap<String, Object>(17);
-        privDict.put("BlueValues", privateDict.getArray("BlueValues", null));
-        privDict.put("OtherBlues", privateDict.getArray("OtherBlues", null));
-        privDict.put("FamilyBlues", privateDict.getArray("FamilyBlues", null));
-        privDict.put("FamilyOtherBlues", privateDict.getArray("FamilyOtherBlues", null));
+        privDict.put("BlueValues", privateDict.getDelta("BlueValues", null));
+        privDict.put("OtherBlues", privateDict.getDelta("OtherBlues", null));
+        privDict.put("FamilyBlues", privateDict.getDelta("FamilyBlues", null));
+        privDict.put("FamilyOtherBlues", privateDict.getDelta("FamilyOtherBlues", null));
         privDict.put("BlueScale", privateDict.getNumber("BlueScale", 0.039625));
         privDict.put("BlueShift", privateDict.getNumber("BlueShift", 7));
         privDict.put("BlueFuzz", privateDict.getNumber("BlueFuzz", 1));
         privDict.put("StdHW", privateDict.getNumber("StdHW", null));
         privDict.put("StdVW", privateDict.getNumber("StdVW", null));
-        privDict.put("StemSnapH", privateDict.getArray("StemSnapH", null));
-        privDict.put("StemSnapV", privateDict.getArray("StemSnapV", null));
+        privDict.put("StemSnapH", privateDict.getDelta("StemSnapH", null));
+        privDict.put("StemSnapV", privateDict.getDelta("StemSnapV", null));
         privDict.put("ForceBold", privateDict.getBoolean("ForceBold", false));
         privDict.put("LanguageGroup", privateDict.getNumber("LanguageGroup", 0));
         privDict.put("ExpansionFactor", privateDict.getNumber("ExpansionFactor", 0.06));
@@ -1156,19 +1173,25 @@ public class CFFParser
         public Boolean getBoolean(String name, boolean defaultValue)
         {
             Entry entry = getEntry(name);
-            return entry != null && entry.getArray().size() > 0 ? entry.getBoolean(0) : defaultValue;
+            return entry != null && !entry.getArray().isEmpty() ? entry.getBoolean(0) : defaultValue;
         }
 
         public List<Number> getArray(String name, List<Number> defaultValue)
         {
             Entry entry = getEntry(name);
-            return entry != null && entry.getArray().size() > 0 ? entry.getArray() : defaultValue;
+            return entry != null && !entry.getArray().isEmpty() ? entry.getArray() : defaultValue;
         }
 
         public Number getNumber(String name, Number defaultValue)
         {
             Entry entry = getEntry(name);
-            return entry != null && entry.getArray().size() > 0 ? entry.getNumber(0) : defaultValue;
+            return entry != null && !entry.getArray().isEmpty() ? entry.getNumber(0) : defaultValue;
+        }
+
+        public List<Number> getDelta(String name, List<Number> defaultValue) 
+        {
+            Entry entry = getEntry(name);
+            return entry != null && !entry.getArray().isEmpty() ? entry.getDelta() : defaultValue;
         }
 
         /**
@@ -1216,6 +1239,19 @@ public class CFFParser
                 return operands;
             }
 
+            public List<Number> getDelta()
+            {
+                List<Number> result = new ArrayList<Number>(operands);
+                for (int i = 1; i < result.size(); i++)
+                {
+                    Number previous = result.get(i - 1);
+                    Number current = result.get(i);
+                    Integer sum = previous.intValue() + current.intValue();
+                    result.set(i, sum);
+                }
+                return result;
+            }
+
             @Override
             public String toString()
             {
