@@ -25,6 +25,8 @@ import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
+import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.io.ScratchFile;
 import org.apache.pdfbox.pdfparser.PDFObjectStreamParser;
 
@@ -83,6 +85,11 @@ public class COSDocument extends COSBase implements Closeable
 
     private ScratchFile scratchFile;
 
+    /**
+     * Used for incremental saving, to avoid XRef object numbers from being reused.
+     */
+    private long highestXRefObjectNumber;
+
     /**
      * Constructor. Uses main memory to buffer PDF streams.
      */
@@ -404,6 +411,28 @@ public class COSDocument extends COSBase implements Closeable
         trailer = newTrailer;
     }
 
+    /**
+     * Internal PDFBox use only. Get the object number of the highest XRef stream. This is needed to
+     * avoid reusing such a number in incremental saving.
+     *
+     * @return The object number of the highest XRef stream, or 0 if there was no XRef stream.
+     */
+    public long getHighestXRefObjectNumber()
+    {
+        return highestXRefObjectNumber;
+    }
+
+    /**
+     * Internal PDFBox use only. Sets the object number of the highest XRef stream. This is needed
+     * to avoid reusing such a number in incremental saving.
+     *
+     * @param highestXRefObjectNumber The object number of the highest XRef stream.
+     */
+    public void setHighestXRefObjectNumber(long highestXRefObjectNumber)
+    {
+        this.highestXRefObjectNumber = highestXRefObjectNumber;
+    }
+
     /**
      * visitor pattern double dispatch method.
      *
@@ -427,24 +456,38 @@ public class COSDocument extends COSBase implements Closeable
     {
         if (!closed)
         {
+            // Make sure that:
+            // - first Exception is kept
+            // - all COSStreams are closed
+            // - ScratchFile is closed
+            // - there's a way to see which errors occured
+
+            IOException firstException = null;
+
             // close all open I/O streams
             for (COSObject object : getObjects())
             {
                 COSBase cosObject = object.getObject();
                 if (cosObject instanceof COSStream)
                 {
-                    ((COSStream) cosObject).close();
+                    firstException = IOUtils.closeAndLogException((COSStream) cosObject, LOG, "COSStream", firstException);
                 }
             }
             for (COSStream stream : streams)
             {
-                stream.close();
+                firstException = IOUtils.closeAndLogException(stream, LOG, "COSStream", firstException);
             }
             if (scratchFile != null)
             {
-                scratchFile.close();
+                firstException = IOUtils.closeAndLogException(scratchFile, LOG, "ScratchFile", firstException);
             }
             closed = true;
+
+            // rethrow first exception to keep method contract
+            if (firstException != null)
+            {
+                throw firstException;
+            }
         }
     }
 
