@@ -17,6 +17,7 @@
 package org.apache.pdfbox.multipdf;
 
 import java.awt.geom.AffineTransform;
+import java.io.Closeable;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -24,8 +25,10 @@ import java.io.OutputStream;
 import java.math.BigDecimal;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -45,7 +48,7 @@ import org.apache.pdfbox.pdmodel.graphics.form.PDFormXObject;
  * Based on code contributed by Balazs Jerk.
  * 
  */
-public class Overlay
+public class Overlay implements Closeable
 {
     /**
      * Possible location of the overlayed pages: foreground or background.
@@ -61,7 +64,7 @@ public class Overlay
     private LayoutPage oddPageOverlayPage;
     private LayoutPage evenPageOverlayPage;
 
-    private final Map<Integer, PDDocument> specificPageOverlay = new HashMap<Integer, PDDocument>();
+    private final Set<PDDocument> openDocuments = new HashSet<PDDocument>();
     private Map<Integer, LayoutPage> specificPageOverlayPage = new HashMap<Integer, LayoutPage>();
 
     private Position position = Position.BACKGROUND;
@@ -87,21 +90,22 @@ public class Overlay
     private String evenPageOverlayFilename = null;
     private PDDocument evenPageOverlay = null;
 
-    
     private int numberOfOverlayPages = 0;
     private boolean useAllOverlayPages = false;
 
     /**
-     * This will add overlays to a documents.
-     * 
-     * @param specificPageOverlayFile map of overlay files for specific pages
-     * 
-     * @return the resulting pdf, which has to be saved and closed be the caller
-     *  
+     * This will add overlays to a document.
+     *
+     * @param specificPageOverlayFile map of overlay files for specific pages. The page numbers are
+     * 1-based.
+     *
+     * @return The modified input PDF document, which has to be saved and closed by the caller. If
+     * the input document was passed by {@link #setInputPDF(PDDocument) setInputPDF(PDDocument)}
+     * then it is that object that is returned.
+     *
      * @throws IOException if something went wrong
      */
-    public PDDocument overlay(Map<Integer, String> specificPageOverlayFile)
-            throws IOException
+    public PDDocument overlay(Map<Integer, String> specificPageOverlayFile) throws IOException
     {
         Map<String, PDDocument> loadedDocuments = new HashMap<String, PDDocument>();
         Map<PDDocument, LayoutPage> layouts = new HashMap<PDDocument, LayoutPage>();
@@ -115,7 +119,7 @@ public class Overlay
                 loadedDocuments.put(e.getValue(), doc);
                 layouts.put(doc, getLayoutPage(doc));
             }
-            specificPageOverlay.put(e.getKey(), doc);
+            openDocuments.add(doc);
             specificPageOverlayPage.put(e.getKey(), layouts.get(doc));
         }
         processPages(inputPDFDocument);
@@ -123,10 +127,38 @@ public class Overlay
     }
 
     /**
-     * Close all input pdfs which were used for the overlay.
-     * 
+     * This will add overlays documents to a document.
+     *
+     * @param specificPageOverlayDocuments map of overlay documents for specific pages. The page
+     * numbers are 1-based.
+     *
+     * @return The modified input PDF document, which has to be saved and closed by the caller. If
+     * the input document was passed by {@link #setInputPDF(PDDocument) setInputPDF(PDDocument)}
+     * then it is that object that is returned.
+     *
      * @throws IOException if something went wrong
      */
+    public PDDocument overlayDocuments(Map<Integer, PDDocument> specificPageOverlayDocuments) throws IOException
+    {
+        loadPDFs();
+        for (Map.Entry<Integer, PDDocument> e : specificPageOverlayDocuments.entrySet())
+        {
+            PDDocument doc = e.getValue();
+            if (doc != null)
+            {
+                specificPageOverlayPage.put(e.getKey(), getLayoutPage(doc));
+            }
+        }
+        processPages(inputPDFDocument);
+        return inputPDFDocument;
+    }
+
+    /**
+     * Close all input documents which were used for the overlay and opened by this class.
+     *
+     * @throws IOException if something went wrong
+     */
+    @Override
     public void close() throws IOException
     {
         if (defaultOverlay != null)
@@ -153,15 +185,12 @@ public class Overlay
         {
             evenPageOverlay.close();
         }
-        if (specificPageOverlay != null)
+        for (PDDocument doc : openDocuments)
         {
-            for (Map.Entry<Integer, PDDocument> e : specificPageOverlay.entrySet())
-            {
-                e.getValue().close();
-            }
-            specificPageOverlay.clear();
-            specificPageOverlayPage.clear();
+            doc.close();
         }
+        openDocuments.clear();
+        specificPageOverlayPage.clear();
     }
 
     private void loadPDFs() throws IOException
@@ -260,7 +289,7 @@ public class Overlay
         {
             resources = new PDResources();
         }
-        return new LayoutPage(page.getMediaBox(), createContentStream(contents),
+        return new LayoutPage(page.getMediaBox(), createCombinedContentStream(contents),
                 resources.getCOSObject());
     }
     
@@ -277,13 +306,13 @@ public class Overlay
             {
                 resources = new PDResources();
             }
-            layoutPages.put(i,new LayoutPage(page.getMediaBox(), createContentStream(contents), 
+            layoutPages.put(i, new LayoutPage(page.getMediaBox(), createCombinedContentStream(contents), 
                     resources.getCOSObject()));
         }
         return layoutPages;
     }
     
-    private COSStream createContentStream(COSBase contents) throws IOException
+    private COSStream createCombinedContentStream(COSBase contents) throws IOException
     {
         List<COSStream> contentStreams = createContentStreamList(contents);
         // concatenate streams
@@ -300,10 +329,15 @@ public class Overlay
         return concatStream;
     }
 
+    // get the content streams as a list
     private List<COSStream> createContentStreamList(COSBase contents) throws IOException
     {
         List<COSStream> contentStreams = new ArrayList<COSStream>();
-        if (contents instanceof COSStream)
+        if (contents == null)
+        {
+            return contentStreams;
+        }
+        else if (contents instanceof COSStream)
         {
             contentStreams.add((COSStream) contents);
         }
@@ -320,45 +354,56 @@ public class Overlay
         }
         else
         {
-            throw new IOException("Contents are unknown type:" + contents.getClass().getName());
+            throw new IOException("Unknown content type: " + contents.getClass().getName());
         }
         return contentStreams;
     }
 
     private void processPages(PDDocument document) throws IOException
     {
-        int pageCount = 0;
+        int pageCounter = 0;
         for (PDPage page : document.getPages())
         {
+            pageCounter++;
             COSDictionary pageDictionary = page.getCOSObject();
-            COSBase contents = pageDictionary.getDictionaryObject(COSName.CONTENTS);
-            COSArray contentArray = new COSArray();
+            COSBase originalContent = pageDictionary.getDictionaryObject(COSName.CONTENTS);
+            COSArray newContentArray = new COSArray();
+            LayoutPage layoutPage = getLayoutPage(pageCounter, document.getNumberOfPages());
+            if (layoutPage == null)
+            {
+                continue;
+            }
             switch (position)
             {
-            case FOREGROUND:
-                // save state
-                contentArray.add(createStream("q\n"));
-                addOriginalContent(contents, contentArray);
-                // restore state
-                contentArray.add(createStream("Q\n"));
-                // overlay content
-                overlayPage(contentArray, page, pageCount + 1, document.getNumberOfPages());
-                break;
-            case BACKGROUND:
-                // overlay content
-                overlayPage(contentArray, page, pageCount + 1, document.getNumberOfPages());
-                addOriginalContent(contents, contentArray);
-                break;
-            default:
-                throw new IOException("Unknown type of position:" + position);
+                case FOREGROUND:
+                    // save state
+                    newContentArray.add(createStream("q\n"));
+                    addOriginalContent(originalContent, newContentArray);
+                    // restore state
+                    newContentArray.add(createStream("Q\n"));
+                    // overlay content last
+                    overlayPage(page, layoutPage, newContentArray);
+                    break;
+                case BACKGROUND:
+                    // overlay content first
+                    overlayPage(page, layoutPage, newContentArray);
+
+                    addOriginalContent(originalContent, newContentArray);
+                    break;
+                default:
+                    throw new IOException("Unknown type of position:" + position);
             }
-            pageDictionary.setItem(COSName.CONTENTS, contentArray);
-            pageCount++;
+            pageDictionary.setItem(COSName.CONTENTS, newContentArray);
         }
     }
 
     private void addOriginalContent(COSBase contents, COSArray contentArray) throws IOException
     {
+        if (contents == null)
+        {
+            return;
+        }
+
         if (contents instanceof COSStream)
         {
             contentArray.add(contents);
@@ -369,12 +414,25 @@ public class Overlay
         }
         else
         {
-            throw new IOException("Unknown content type:" + contents.getClass().getName());
+            throw new IOException("Unknown content type: " + contents.getClass().getName());
         }
     }
 
-    private void overlayPage(COSArray array, PDPage page, int pageNumber, int numberOfPages)
+    private void overlayPage(PDPage page, LayoutPage layoutPage, COSArray array)
             throws IOException
+    {
+        PDResources resources = page.getResources();
+        if (resources == null)
+        {
+            resources = new PDResources();
+            page.setResources(resources);
+        }
+        COSName xObjectId = createOverlayXObject(page, layoutPage,
+                layoutPage.overlayContentStream);
+        array.add(createOverlayStream(page, layoutPage, xObjectId));
+    }
+
+    private LayoutPage getLayoutPage(int pageNumber, int numberOfPages)
     {
         LayoutPage layoutPage = null;
         if (!useAllOverlayPages && specificPageOverlayPage.containsKey(pageNumber))
@@ -406,18 +464,7 @@ public class Overlay
             int usePageNum = (pageNumber -1 ) % numberOfOverlayPages;
             layoutPage = specificPageOverlayPage.get(usePageNum);
         }
-        if (layoutPage != null)
-        {
-            PDResources resources = page.getResources();
-            if (resources == null)
-            {
-                resources = new PDResources();
-                page.setResources(resources);
-            }
-            COSName xObjectId = createOverlayXObject(page, layoutPage,
-                    layoutPage.overlayContentStream);
-            array.add(createOverlayStream(page, layoutPage, xObjectId));
-        }
+        return layoutPage;
     }
 
     private COSName createOverlayXObject(PDPage page, LayoutPage layoutPage, COSStream contentStream)
@@ -435,20 +482,41 @@ public class Overlay
             throws IOException
     {
         // create a new content stream that executes the XObject content
-        PDRectangle pageMediaBox = page.getMediaBox();
-        float hShift = (pageMediaBox.getWidth() - layoutPage.overlayMediaBox.getWidth()) / 2.0f;
-        float vShift = (pageMediaBox.getHeight() - layoutPage.overlayMediaBox.getHeight()) / 2.0f;
         StringBuilder overlayStream = new StringBuilder();
-        overlayStream.append("q\nq 1 0 0 1 ");
-        overlayStream.append(float2String(hShift));
-        overlayStream.append(" ");
-        overlayStream.append(float2String(vShift) );
-        overlayStream.append(" cm /");
+        overlayStream.append("q\nq\n");
+        AffineTransform at = calculateAffineTransform(page, layoutPage.overlayMediaBox);
+        double[] flatmatrix = new double[6];
+        at.getMatrix(flatmatrix);
+        for (double v : flatmatrix)
+        {
+            overlayStream.append(float2String((float) v));
+            overlayStream.append(" ");
+        }
+        overlayStream.append(" cm\n/");
         overlayStream.append(xObjectId.getName());
         overlayStream.append(" Do Q\nQ\n");
         return createStream(overlayStream.toString());
     }
 
+    /**
+     * Calculate the transform to be used when positioning the overlay. The default implementation
+     * centers on the destination. Override this method to do your own, e.g. move to a corner, or
+     * rotate.
+     *
+     * @param page The page that will get the overlay.
+     * @param overlayMediaBox The overlay media box.
+     * @return The affine transform to be used.
+     */
+    protected AffineTransform calculateAffineTransform(PDPage page, PDRectangle overlayMediaBox)
+    {
+        AffineTransform at = new AffineTransform();
+        PDRectangle pageMediaBox = page.getMediaBox();
+        float hShift = (pageMediaBox.getWidth() - overlayMediaBox.getWidth()) / 2.0f;
+        float vShift = (pageMediaBox.getHeight() - overlayMediaBox.getHeight()) / 2.0f;
+        at.translate(hShift, vShift);
+        return at;
+    }
+
     private String float2String(float floatValue)
     {
         // use a BigDecimal as intermediate state to avoid 
@@ -488,8 +556,10 @@ public class Overlay
 
     /**
      * Sets the file to be overlayed.
-     * 
-     * @param inputFile the file to be overlayed
+     *
+     * @param inputFile the file to be overlayed. The {@link PDDocument} object gathered from
+     * opening this file will be returned by
+     * {@link #overlay(java.util.Map) overlay(Map&lt;Integer, String&gt;)}.
      */
     public void setInputFile(String inputFile)
     {
@@ -498,8 +568,9 @@ public class Overlay
 
     /**
      * Sets the PDF to be overlayed.
-     * 
-     * @param inputPDF the PDF to be overlayed
+     *
+     * @param inputPDF the PDF to be overlayed. This will be the object that is returned by
+     * {@link #overlay(java.util.Map) overlay(Map&lt;Integer, String&gt;)}.
      */
     public void setInputPDF(PDDocument inputPDF)
     {
