@@ -31,13 +31,18 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSInteger;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSNumber;
+import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
+import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.io.MemoryUsageSetting;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
@@ -71,21 +76,47 @@ import org.apache.pdfbox.pdmodel.interactive.form.PDField;
  */
 public class PDFMergerUtility
 {
-    private final List<InputStream> sources;
-    private final List<FileInputStream> fileInputStreams;
+    /**
+     * Log instance.
+     */
+    private static final Log LOG = LogFactory.getLog(PDFMergerUtility.class);
+
+    private final List<Object> sources;
     private String destinationFileName;
     private OutputStream destinationStream;
     private boolean ignoreAcroFormErrors = false;
     private PDDocumentInformation destinationDocumentInformation = null;
     private PDMetadata destinationMetadata = null;
 
+    private DocumentMergeMode documentMergeMode = DocumentMergeMode.PDFBOX_LEGACY_MODE;
+
+    /**
+     * The mode to use when merging documents:
+     * 
+     * <ul>
+     * <li>{@link DocumentMergeMode#OPTIMIZE_RESOURCES_MODE} Optimizes resource handling such as
+     *      closing documents early. <strong>Not all document elements are merged</strong> compared to
+     *      the PDFBOX_LEGACY_MODE. Currently supported are:
+     *      <ul>
+     *          <li>Page content and resources
+     *      </ul>  
+     * <li>{@link DocumentMergeMode#PDFBOX_LEGACY_MODE} Keeps all files open until the
+     *      merge has been completed. This is  currently necessary to merge documents
+     *      containing a Structure Tree. <br>This is the standard mode for PDFBox 2.0.
+     * </ul>
+     */
+    public enum DocumentMergeMode
+    {
+        OPTIMIZE_RESOURCES_MODE,
+        PDFBOX_LEGACY_MODE
+    }
+
     /**
      * Instantiate a new PDFMergerUtility.
      */
     public PDFMergerUtility()
     {
-        sources = new ArrayList<InputStream>();
-        fileInputStreams = new ArrayList<FileInputStream>();
+        sources = new ArrayList<Object>();
     }
 
     /**
@@ -193,9 +224,7 @@ public class PDFMergerUtility
      */
     public void addSource(File source) throws FileNotFoundException
     {
-        FileInputStream stream = new FileInputStream(source);
-        sources.add(stream);
-        fileInputStreams.add(stream);
+        sources.add(source);
     }
 
     /**
@@ -241,12 +270,101 @@ public class PDFMergerUtility
      * @throws IOException If there is an error saving the document.
      */
     public void mergeDocuments(MemoryUsageSetting memUsageSetting) throws IOException
+    {
+        if (documentMergeMode == DocumentMergeMode.PDFBOX_LEGACY_MODE)
+        {
+            legacyMergeDocuments(memUsageSetting);
+        }
+        else if (documentMergeMode == DocumentMergeMode.OPTIMIZE_RESOURCES_MODE)
+        {
+            optimizedMergeDocuments(memUsageSetting, sources);
+        }
+    }
+
+    private void optimizedMergeDocuments(MemoryUsageSetting memUsageSetting,
+            List<Object> sourceDocuments) throws IOException
+    {
+        PDDocument destination = null;
+        try
+        {
+            destination = new PDDocument(memUsageSetting);
+            PDFCloneUtility cloner = new PDFCloneUtility(destination);
+
+            for (Object sourceObject : sources)
+            {
+                PDDocument sourceDoc = null;
+                try
+                {
+                    if (sourceObject instanceof File)
+                    {
+                        sourceDoc = PDDocument.load((File) sourceObject, memUsageSetting);
+                    }
+                    else
+                    {
+                        sourceDoc = PDDocument.load((InputStream) sourceObject, memUsageSetting);
+                    }
+
+                    for (PDPage page : sourceDoc.getPages())
+                    {
+                        PDPage newPage = new PDPage((COSDictionary) cloner.cloneForNewDocument(page.getCOSObject()));
+                        newPage.setCropBox(page.getCropBox());
+                        newPage.setMediaBox(page.getMediaBox());
+                        newPage.setRotation(page.getRotation());
+                        PDResources resources = page.getResources();
+                        if (resources != null)
+                        {
+                            // this is smart enough to just create references for resources that are used on multiple pages
+                            newPage.setResources(new PDResources((COSDictionary) cloner.cloneForNewDocument(resources)));
+                        }
+                        else
+                        {
+                            newPage.setResources(new PDResources());
+                        }
+                        destination.addPage(newPage);
+                    }
+                }
+                finally
+                {
+                    IOUtils.closeQuietly(sourceDoc);
+                }
+            }
+            
+            if (destinationStream == null)
+            {
+                destination.save(destinationFileName);
+            }
+            else
+            {
+                destination.save(destinationStream);
+            }
+        }
+        finally
+        {
+            IOUtils.closeQuietly(destination);
+        }
+    }
+
+    /**
+     * Merge the list of source documents, saving the result in the destination
+     * file.
+     *
+     * @param memUsageSetting defines how memory is used for buffering PDF streams;
+     *                        in case of <code>null</code> unrestricted main memory is used 
+     * 
+     * @throws IOException If there is an error saving the document.
+     */
+    private void legacyMergeDocuments(MemoryUsageSetting memUsageSetting) throws IOException
     {
         PDDocument destination = null;
-        InputStream sourceFile;
-        PDDocument source;
         if (sources != null && sources.size() > 0)
         {
+            // Make sure that:
+            // - first Exception is kept
+            // - destination is closed
+            // - all PDDocuments are closed
+            // - all FileInputStreams are closed
+            // - there's a way to see which errors occured
+
             List<PDDocument> tobeclosed = new ArrayList<PDDocument>();
 
             try
@@ -256,9 +374,18 @@ public class PDFMergerUtility
                         MemoryUsageSetting.setupMainMemoryOnly();
                 destination = new PDDocument(partitionedMemSetting);
 
-                for (InputStream sourceInputStream : sources)
+                for (Object sourceObject : sources)
                 {
-                    PDDocument sourceDoc = PDDocument.load(sourceInputStream, partitionedMemSetting);
+                    PDDocument sourceDoc = null;
+                    if (sourceObject instanceof File)
+                    {
+                        sourceDoc = PDDocument.load((File) sourceObject, partitionedMemSetting);
+                    }
+                    else
+                    {
+                        sourceDoc = PDDocument.load((InputStream) sourceObject,
+                                partitionedMemSetting);
+                    }
                     tobeclosed.add(sourceDoc);
                     appendDocument(destination, sourceDoc);
                 }
@@ -286,15 +413,12 @@ public class PDFMergerUtility
             {
                 if (destination != null)
                 {
-                    destination.close();
+                    IOUtils.closeAndLogException(destination, LOG, "PDDocument", null);
                 }
+
                 for (PDDocument doc : tobeclosed)
                 {
-                    doc.close();
-                }
-                for (FileInputStream stream : fileInputStreams)
-                {
-                    stream.close();
+                    IOUtils.closeAndLogException(doc, LOG, "PDDocument", null);
                 }
             }
         }
@@ -345,7 +469,16 @@ public class PDFMergerUtility
         if (destCatalog.getOpenAction() == null)
         {
             // PDFBOX-3972: get local dest page index, it must be reassigned after the page cloning
-            PDDestinationOrAction openAction = srcCatalog.getOpenAction();
+            PDDestinationOrAction openAction = null;
+            try
+            {
+                openAction = srcCatalog.getOpenAction();
+            }
+            catch (IOException ex)
+            {
+                // PDFBOX-4223
+                LOG.error("Invalid OpenAction ignored", ex);
+            }
             PDDestination openActionDestination = null;
             if (openAction instanceof PDActionGoTo)
             {
@@ -366,7 +499,7 @@ public class PDFMergerUtility
                 }
             }
 
-            destCatalog.setOpenAction(srcCatalog.getOpenAction());
+            destCatalog.setOpenAction(openAction);
         }
 
         PDFCloneUtility cloner = new PDFCloneUtility(destination);
@@ -499,9 +632,21 @@ public class PDFMergerUtility
             COSArray srcNums = (COSArray) srcLabels.getDictionaryObject(COSName.NUMS);
             if (srcNums != null)
             {
+                int startSize = destNums.size();
                 for (int i = 0; i < srcNums.size(); i += 2)
                 {
-                    COSNumber labelIndex = (COSNumber) srcNums.getObject(i);
+                    COSBase base = srcNums.getObject(i);
+                    if (!(base instanceof COSNumber))
+                    {
+                        LOG.error("page labels ignored, index " + i + " should be a number, but is " + base);
+                        // remove what we added
+                        while (destNums.size() > startSize)
+                        {
+                            destNums.remove(startSize);
+                        }
+                        break;
+                    }
+                    COSNumber labelIndex = (COSNumber) base;
                     long labelIndexValue = labelIndex.intValue();
                     destNums.add(COSInteger.get(labelIndexValue + destPageCount));
                     destNums.add(cloner.cloneForNewDocument(srcNums.getObject(i + 1)));
@@ -513,10 +658,18 @@ public class PDFMergerUtility
         COSStream srcMetadata = (COSStream) srcCatalog.getCOSObject().getDictionaryObject(COSName.METADATA);
         if (destMetadata == null && srcMetadata != null)
         {
-            PDStream newStream = new PDStream(destination, srcMetadata.createInputStream(), (COSName) null);           
-            mergeInto(srcMetadata, newStream.getCOSObject(), 
-                    new HashSet<COSName>(Arrays.asList(COSName.FILTER, COSName.LENGTH)));           
-            destCatalog.getCOSObject().setItem(COSName.METADATA, newStream);
+            try
+            {
+                PDStream newStream = new PDStream(destination, srcMetadata.createInputStream(), (COSName) null);           
+                mergeInto(srcMetadata, newStream.getCOSObject(), 
+                        new HashSet<COSName>(Arrays.asList(COSName.FILTER, COSName.LENGTH)));           
+                destCatalog.getCOSObject().setItem(COSName.METADATA, newStream);
+            }
+            catch (IOException ex)
+            {
+                // PDFBOX-4227 cleartext XMP stream with /Flate 
+                LOG.error("Metadata skipped because it could not be read", ex);
+            }
         }
 
         COSDictionary destOCP = (COSDictionary) destCatalog.getCOSObject().getDictionaryObject(COSName.OCPROPERTIES);
@@ -599,6 +752,8 @@ public class PDFMergerUtility
             }
             if (mergeStructTree)
             {
+                // add the value of the destination ParentTreeNextKey to every source element 
+                // StructParent(s) value so that these don't overlap with the existing values
                 updateStructParentEntries(newPage, destParentTreeNextKey);
                 objMapping.put(page.getCOSObject(), newPage.getCOSObject());
                 List<PDAnnotation> oldAnnots = page.getAnnotations();
@@ -631,7 +786,7 @@ public class PDFMergerUtility
         }
         if (mergeStructTree)
         {
-            updatePageReferences(srcNumbersArray, objMapping);
+            updatePageReferences(cloner, srcNumbersArray, objMapping);
             for (int i = 0; i < srcNumbersArray.size() / 2; i++)
             {
                 destNumbersArray.add(COSInteger.get(destParentTreeNextKey + i));
@@ -781,7 +936,9 @@ public class PDFMergerUtility
      * @param parentTreeEntry
      * @param objMapping mapping between old and new references
      */
-    private void updatePageReferences(COSDictionary parentTreeEntry, Map<COSDictionary, COSDictionary> objMapping)
+    private void updatePageReferences(PDFCloneUtility cloner,
+            COSDictionary parentTreeEntry, Map<COSDictionary, COSDictionary> objMapping)
+            throws IOException
     {
         COSBase page = parentTreeEntry.getDictionaryObject(COSName.PG);
         if (page instanceof COSDictionary && objMapping.containsKey(page))
@@ -789,33 +946,56 @@ public class PDFMergerUtility
             parentTreeEntry.setItem(COSName.PG, objMapping.get(page));
         }
         COSBase obj = parentTreeEntry.getDictionaryObject(COSName.OBJ);
-        if (obj instanceof COSDictionary && objMapping.containsKey(obj))
+        if (obj instanceof COSDictionary)
         {
-            parentTreeEntry.setItem(COSName.OBJ, objMapping.get(obj));
+            if (objMapping.containsKey(obj))
+            {
+                parentTreeEntry.setItem(COSName.OBJ, objMapping.get(obj));
+            }
+            else
+            {
+                // PDFBOX-3999: clone objects that are not in mapping to make sure that
+                // these don't remain attached to the source document
+                COSBase item = parentTreeEntry.getItem(COSName.OBJ);
+                if (item instanceof COSObject)
+                {
+                    LOG.debug("clone potential orphan object in structure tree: " + item +
+                            ", type: " + ((COSDictionary) obj).getNameAsString(COSName.TYPE));
+                }
+                else
+                {
+                    // don't display because of stack overflow
+                    LOG.debug("clone potential orphan object in structure tree, type: " +
+                            ((COSDictionary) obj).getNameAsString(COSName.TYPE));
+                }
+                parentTreeEntry.setItem(COSName.OBJ, cloner.cloneForNewDocument(obj));
+            }
         }
         COSBase kSubEntry = parentTreeEntry.getDictionaryObject(COSName.K);
         if (kSubEntry instanceof COSArray)
         {
-            updatePageReferences((COSArray) kSubEntry, objMapping);
+            updatePageReferences(cloner, (COSArray) kSubEntry, objMapping);
         }
         else if (kSubEntry instanceof COSDictionary)
         {
-            updatePageReferences((COSDictionary) kSubEntry, objMapping);
+            updatePageReferences(cloner, (COSDictionary) kSubEntry, objMapping);
         }
     }
 
-    private void updatePageReferences(COSArray parentTreeEntry, Map<COSDictionary, COSDictionary> objMapping)
+    private void updatePageReferences(PDFCloneUtility cloner,
+            COSArray parentTreeEntry, Map<COSDictionary, COSDictionary> objMapping)
+            throws IOException
     {
         for (int i = 0; i < parentTreeEntry.size(); i++)
         {
             COSBase subEntry = parentTreeEntry.getObject(i);
             if (subEntry instanceof COSArray)
             {
-                updatePageReferences((COSArray) subEntry, objMapping);
+                updatePageReferences(cloner, (COSArray) subEntry, objMapping);
             }
             else if (subEntry instanceof COSDictionary)
             {
-                updatePageReferences((COSDictionary) subEntry, objMapping);
+                updatePageReferences(cloner, (COSDictionary) subEntry, objMapping);
             }
         }
     }
@@ -873,9 +1053,10 @@ public class PDFMergerUtility
     }
 
     /**
-     * This will add all of the dictionarys keys/values to this dictionary, but only if they are not
-     * in an exclusion list and if they don't already exist. If a key already exists in this
-     * dictionary then nothing is changed.
+     * This will add all of the dictionaries keys/values to this dictionary, but
+     * only if they are not in an exclusion list and if they don't already
+     * exist. If a key already exists in this dictionary then nothing is
+     * changed.
      *
      * @param src The source dictionary to get the keys/values from.
      * @param dst The destination dictionary to merge the keys/values into.
