@@ -16,7 +16,6 @@
  */
 package org.apache.pdfbox.pdfwriter;
 
-import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.Closeable;
 import java.io.IOException;
@@ -221,22 +220,24 @@ public class COSWriter implements ICOSVisitor, Closeable
     private COSArray byteRangeArray;
 
     /**
-     * COSWriter constructor comment.
+     * COSWriter constructor.
      *
-     * @param os The wrapped output stream.
+     * @param outputStream The output stream to write the PDF. It will be closed when this object is
+     * closed.
      */
-    public COSWriter(OutputStream os)
+    public COSWriter(OutputStream outputStream)
     {
-        setOutput(os);
+        setOutput(outputStream);
         setStandardOutput(new COSStandardOutputStream(output));
     }
 
     /**
-     * COSWriter constructor for incremental updates. 
+     * COSWriter constructor for incremental updates.
      *
-     * @param outputStream output stream where the new PDF data will be written
+     * @param outputStream output stream where the new PDF data will be written. It will be closed
+     * when this object is closed.
      * @param inputData random access read containing source PDF data
-     * 
+     *
      * @throws IOException if something went wrong
      */
     public COSWriter(OutputStream outputStream, RandomAccessRead inputData) throws IOException
@@ -260,7 +261,7 @@ public class COSWriter implements ICOSVisitor, Closeable
           
           Map<COSObjectKey, Long> xrefTable = cosDoc.getXrefTable();
           Set<COSObjectKey> keySet = xrefTable.keySet();
-          long highestNumber=0;
+          long highestNumber = doc.getDocument().getHighestXRefObjectNumber();
           for ( COSObjectKey cosObjectKey : keySet ) 
           {
             COSBase object = cosDoc.getObjectFromPool(cosObjectKey).getObject();
@@ -310,10 +311,6 @@ public class COSWriter implements ICOSVisitor, Closeable
         {
             getStandardOutput().close();
         }
-        if (getOutput() != null)
-        {
-            getOutput().close();
-        }
         if (incrementalOutput != null)
         {
             incrementalOutput.close();
@@ -670,19 +667,17 @@ public class COSWriter implements ICOSVisitor, Closeable
     }
 
     /**
-     * Write an incremental update for a non signature case. This can be used for e.g. augmenting signatures.
-     * 
+     * Write an incremental update for a non signature case. This can be used for e.g. augmenting
+     * signatures.
+     *
      * @throws IOException
      */
     private void doWriteIncrement() throws IOException
     {
-        ByteArrayOutputStream byteOut = (ByteArrayOutputStream) output;
-        byteOut.flush();
-        byte[] buffer = byteOut.toByteArray();
-        SequenceInputStream signStream = new SequenceInputStream(new RandomAccessInputStream(incrementalInput),
-                                                                 new ByteArrayInputStream(buffer));
-        // write the data to the incremental output stream
-        IOUtils.copy(signStream, incrementalOutput);
+        // write existing PDF
+        IOUtils.copy(new RandomAccessInputStream(incrementalInput), incrementalOutput);
+        // write the actual incremental update
+        incrementalOutput.write(((ByteArrayOutputStream) output).toByteArray());
     }
     
     private void doWriteSignature() throws IOException
@@ -933,8 +928,12 @@ public class COSWriter implements ICOSVisitor, Closeable
             else if( current instanceof COSObject )
             {
                 COSBase subValue = ((COSObject)current).getObject();
-                if (incrementalUpdate || subValue instanceof COSDictionary || subValue == null)
+                if (willEncrypt || incrementalUpdate || subValue instanceof COSDictionary || subValue == null)
                 {
+                    // PDFBOX-4308: added willEncrypt to prevent an object
+                    // that is referenced several times from being written
+                    // direct and indirect, thus getting encrypted
+                    // with wrong object number or getting encrypted twice
                     addObjectToWrite( current );
                     writeReference( current );
                 }
@@ -1031,8 +1030,12 @@ public class COSWriter implements ICOSVisitor, Closeable
                 else if( value instanceof COSObject )
                 {
                     COSBase subValue = ((COSObject)value).getObject();
-                    if (incrementalUpdate || subValue instanceof COSDictionary || subValue == null)
+                    if (willEncrypt || incrementalUpdate || subValue instanceof COSDictionary || subValue == null)
                     {
+                        // PDFBOX-4308: added willEncrypt to prevent an object
+                        // that is referenced several times from being written
+                        // direct and indirect, thus getting encrypted
+                        // with wrong object number or getting encrypted twice
                         addObjectToWrite( value );
                         writeReference( value );
                     }
@@ -1125,14 +1128,16 @@ public class COSWriter implements ICOSVisitor, Closeable
         getStandardOutput().write(EOF);
         getStandardOutput().writeEOL();
 
-        if(incrementalUpdate)
+        if (incrementalUpdate)
         {
-          if (signatureOffset == 0 || byteRangeOffset == 0)
-          {
-              doWriteIncrement();
-          } else {
-              doWriteSignature();
-          }
+            if (signatureOffset == 0 || byteRangeOffset == 0)
+            {
+                doWriteIncrement();
+            }
+            else
+            {
+                doWriteSignature();
+            }
         }
 
         return null;
