@@ -22,7 +22,6 @@ import java.util.List;
 
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
-import org.apache.pdfbox.cos.COSBoolean;
 import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSObject;
@@ -161,8 +160,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDDocumentOutline getDocumentOutline()
     {
-        COSDictionary dict = (COSDictionary)root.getDictionaryObject(COSName.OUTLINES);
-        return dict == null ? null : new PDDocumentOutline(dict);
+        COSBase cosObj = root.getDictionaryObject(COSName.OUTLINES);
+        return cosObj instanceof COSDictionary ? new PDDocumentOutline((COSDictionary)cosObj) : null;
     }
 
     /**
@@ -251,22 +250,7 @@ public class PDDocumentCatalog implements COSObjectable
     public PDDestinationOrAction getOpenAction() throws IOException
     {
         COSBase openAction = root.getDictionaryObject(COSName.OPEN_ACTION);
-        if (openAction == null)
-        {
-            return null;
-        }
-        else if (openAction instanceof COSBoolean)
-        {
-            if (((COSBoolean) openAction).getValue() == false)
-            {
-                return null;
-            }
-            else
-            {
-                throw new IOException("Can't create OpenAction from COSBoolean");
-            }
-        }
-        else if (openAction instanceof COSDictionary)
+        if (openAction instanceof COSDictionary)
         {
             return PDActionFactory.createAction((COSDictionary)openAction);
         }
@@ -276,7 +260,7 @@ public class PDDocumentCatalog implements COSObjectable
         }
         else
         {
-            throw new IOException("Unknown OpenAction " + openAction);
+            return null;
         }
     }
     /**
