@@ -465,6 +465,7 @@ public final class PDPageContentStream implements Closeable
      *
      * @param text The Unicode text to show.
      * @throws IOException If an io exception occurs.
+     * @throws IllegalArgumentException if a character isn't supported by the current font
      */
     public void showText(String text) throws IOException
     {
@@ -2502,6 +2503,10 @@ public final class PDPageContentStream implements Closeable
     @Override
     public void close() throws IOException
     {
+        if (inTextMode)
+        {
+            LOG.warn("You did not call endText(), some viewers won't display your text");
+        }
         if (output != null)
         {
             output.close();
