@@ -315,7 +315,8 @@ public abstract class PDFont implements COSObjectable, PDFontLike
     public final byte[] encode(String text) throws IOException
     {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        for (int offset = 0; offset < text.length(); )
+        int offset = 0;
+        while (offset < text.length())
         {
             int codePoint = text.codePointAt(offset);
 
@@ -439,13 +440,16 @@ public abstract class PDFont implements COSObjectable, PDFontLike
         // if the font dictionary containsName a ToUnicode CMap, use that CMap
         if (toUnicodeCMap != null)
         {
-            if (toUnicodeCMap.getName() != null && toUnicodeCMap.getName().startsWith("Identity-") && 
-                    dict.getDictionaryObject(COSName.TO_UNICODE) instanceof COSName)
+            if (toUnicodeCMap.getName() != null && 
+                toUnicodeCMap.getName().startsWith("Identity-") && 
+                    (dict.getDictionaryObject(COSName.TO_UNICODE) instanceof COSName ||
+                     !toUnicodeCMap.hasUnicodeMappings()))
             {
                 // handle the undocumented case of using Identity-H/V as a ToUnicode CMap, this
                 // isn't actually valid as the Identity-x CMaps are code->CID maps, not
                 // code->Unicode maps. See sample_fonts_solidconvertor.pdf for an example.
                 // PDFBOX-3123: do this only if the /ToUnicode entry is a name
+                // PDFBOX-4322: identity streams are OK too
                 return new String(new char[] { (char) code });
             }
             else
