@@ -18,7 +18,6 @@ package org.apache.pdfbox.pdmodel.font;
 
 import java.awt.geom.GeneralPath;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.HashMap;
@@ -309,7 +308,7 @@ public class PDTrueTypeFont extends PDSimpleFont implements PDVectorFont
         glyphList = GlyphList.getAdobeGlyphList();
         if (closeTTF)
         {
-            // the TTF is fully loaded and it is save to close the underlying data source
+            // the TTF is fully loaded and it is safe to close the underlying data source
             ttf.close();
         }
     }
