@@ -194,8 +194,20 @@ public class PDType0Font extends PDFont implements PDVectorFont
         fetchCMapUCS2();
     }
 
+    /**
+     * Private. Creates a new PDType0Font font for embedding.
+     *
+     * @param document
+     * @param ttf
+     * @param embedSubset
+     * @param closeTTF whether to close the ttf parameter after embedding. Must be true when the ttf
+     * parameter was created in the load() method, false when the ttf parameter was passed to the
+     * load() method.
+     * @param vertical
+     * @throws IOException
+     */
     private PDType0Font(PDDocument document, TrueTypeFont ttf, boolean embedSubset,
-            boolean closeOnSubset, boolean vertical) throws IOException
+            boolean closeTTF, boolean vertical) throws IOException
     {
         if (vertical)
         {
@@ -205,15 +217,16 @@ public class PDType0Font extends PDFont implements PDVectorFont
         descendantFont = embedder.getCIDFont();
         readEncoding();
         fetchCMapUCS2();
-        if (closeOnSubset)
+        if (closeTTF)
         {
             if (embedSubset)
             {
                 this.ttf = ttf;
+                document.registerTrueTypeFontForClosing(ttf);
             }
             else
             {
-                // the TTF is fully loaded and it is save to close the underlying data source
+                // the TTF is fully loaded and it is safe to close the underlying data source
                 ttf.close();
             }
         }
@@ -550,7 +563,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
         {
             descendant = getDescendantFont().getClass().getSimpleName();
         }
-        return getClass().getSimpleName() + "/" + descendant + " " + getBaseFont();
+        return getClass().getSimpleName() + "/" + descendant + ", PostScript name: " + getBaseFont();
     }
 
     @Override
