@@ -25,6 +25,7 @@ import org.apache.commons.logging.LogFactory;
 import java.io.IOException;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
 /**
  * PostScript glyph list, maps glyph names to sequences of Unicode characters.
@@ -96,7 +97,7 @@ public final class GlyphList
     private final Map<String, String> unicodeToName;
     
     // additional read/write cache for uniXXXX names
-    private final Map<String, String> uniNameToUnicodeCache = new HashMap<String, String>();
+    private final Map<String, String> uniNameToUnicodeCache = new ConcurrentHashMap<String, String>();
 
     /**
      * Creates a new GlyphList from a glyph list file.
@@ -290,7 +291,11 @@ public final class GlyphList
                     LOG.warn("Not a number in Unicode character name: " + name);
                 }
             }
-            uniNameToUnicodeCache.put(name, unicode);
+            if (unicode != null)
+            {
+                // null value not allowed in ConcurrentHashMap
+                uniNameToUnicodeCache.put(name, unicode);
+            }
         }
         return unicode;
     }
