@@ -292,14 +292,25 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
         if (fileType.equals(FileType.TIFF))
         {
-            return CCITTFactory.createFromFile(doc, file);
+            try
+            {
+                return CCITTFactory.createFromFile(doc, file);
+            }
+            catch (IOException ex)
+            {
+                LOG.debug("Reading as TIFF failed, setting fileType to PNG", ex);
+                // Plan B: try reading with ImageIO
+                // common exception:
+                // First image in tiff is not CCITT T4 or T6 compressed
+                fileType = FileType.PNG;
+            }
         }
         if (fileType.equals(FileType.BMP) || fileType.equals(FileType.GIF) || fileType.equals(FileType.PNG))
         {
             BufferedImage bim = ImageIO.read(file);
             return LosslessFactory.createFromImage(doc, bim);
         }
-        throw new IllegalArgumentException("Image type not supported: " + file.getName());
+        throw new IllegalArgumentException("Image type " + fileType + " not supported: " + file.getName());
     }
 
     /**
@@ -340,7 +351,18 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
         if (fileType.equals(FileType.TIFF))
         {
-            return CCITTFactory.createFromByteArray(document, byteArray);
+            try
+            {
+                return CCITTFactory.createFromByteArray(document, byteArray);
+            }
+            catch (IOException ex)
+            {
+                LOG.debug("Reading as TIFF failed, setting fileType to PNG", ex);
+                // Plan B: try reading with ImageIO
+                // common exception:
+                // First image in tiff is not CCITT T4 or T6 compressed
+                fileType = FileType.PNG;
+            }
         }
         if (fileType.equals(FileType.BMP) || fileType.equals(FileType.GIF) || fileType.equals(FileType.PNG))
         {
@@ -348,7 +370,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
             BufferedImage bim = ImageIO.read(bais);
             return LosslessFactory.createFromImage(document, bim);
         }
-        throw new IllegalArgumentException("Image type not supported: " + name);
+        throw new IllegalArgumentException("Image type " + fileType + " not supported: " + name);
     }
 
     /**
@@ -424,7 +446,8 @@ public final class PDImageXObject extends PDXObject implements PDImage
         PDImageXObject softMask = getSoftMask();
         if (softMask != null)
         {
-            image = applyMask(image, softMask.getOpaqueImage(), true);
+            float[] matte = extractMatte(softMask);
+            image = applyMask(image, softMask.getOpaqueImage(), true, matte);
         }
         else
         {
@@ -432,7 +455,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
             PDImageXObject mask = getMask();
             if (mask != null && mask.isStencil())
             {
-                image = applyMask(image, mask.getOpaqueImage(), false);
+                image = applyMask(image, mask.getOpaqueImage(), false, null);
             }
         }
 
@@ -447,6 +470,21 @@ public final class PDImageXObject extends PDXObject implements PDImage
         return image;
     }
 
+    private float[] extractMatte(PDImageXObject softMask) throws IOException
+    {
+        COSBase base = softMask.getCOSObject().getItem(COSName.MATTE);
+        float[] matte = null;
+        if (base instanceof COSArray)
+        {
+            // PDFBOX-4267: process /Matte
+            // see PDF specification 1.7, 11.6.5.3 Soft-Mask Images
+            matte = ((COSArray) base).toFloatArray();
+            // convert to RGB
+            matte = getColorSpace().toRGB(matte);
+        }
+        return matte;
+    }
+
     /**
      * {@inheritDoc}
      * The returned images are not cached.
@@ -474,7 +512,8 @@ public final class PDImageXObject extends PDXObject implements PDImage
 
     // explicit mask: RGB + Binary -> ARGB
     // soft mask: RGB + Gray -> ARGB
-    private BufferedImage applyMask(BufferedImage image, BufferedImage mask, boolean isSoft)
+    private BufferedImage applyMask(BufferedImage image, BufferedImage mask,
+                                    boolean isSoft, float[] matte)
             throws IOException
     {
         if (mask == null)
@@ -520,6 +559,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
                 if (isSoft)
                 {
                     rgba[3] = alphaPixel[0];
+                    if (matte != null && Float.compare(alphaPixel[0], 0) != 0)
+                    {
+                        rgba[0] = clampColor(((rgba[0] / 255 - matte[0]) / (alphaPixel[0] / 255) + matte[0]) * 255);
+                        rgba[1] = clampColor(((rgba[1] / 255 - matte[1]) / (alphaPixel[0] / 255) + matte[1]) * 255);
+                        rgba[2] = clampColor(((rgba[2] / 255 - matte[2]) / (alphaPixel[0] / 255) + matte[2]) * 255);
+                    }
                 }
                 else
                 {
@@ -533,6 +578,11 @@ public final class PDImageXObject extends PDXObject implements PDImage
         return masked;
     }
 
+    private float clampColor(float color)
+    {
+        return color < 0 ? 0 : (color > 255 ? 255 : color);        
+    }
+
     /**
      * High-quality image scaling.
      */
@@ -540,10 +590,13 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         BufferedImage image2 = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
         Graphics2D g = image2.createGraphics();
-        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
-                           RenderingHints.VALUE_INTERPOLATION_BICUBIC);
-        g.setRenderingHint(RenderingHints.KEY_RENDERING,
-                           RenderingHints.VALUE_RENDER_QUALITY);
+        if (getInterpolate())
+        {
+            g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+                    RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+            g.setRenderingHint(RenderingHints.KEY_RENDERING,
+                    RenderingHints.VALUE_RENDER_QUALITY);
+        }
         g.drawImage(image, 0, 0, width, height, 0, 0, image.getWidth(), image.getHeight(), null);
         g.dispose();
         return image2;
