@@ -346,13 +346,13 @@ public final class PDInlineImage implements PDImage
     @Override
     public BufferedImage getImage() throws IOException
     {
-        return SampledImageReader.getRGBImage(this, getColorKeyMask());
+        return SampledImageReader.getRGBImage(this, null);
     }
 
     @Override
     public BufferedImage getImage(Rectangle region, int subsampling) throws IOException
     {
-        return SampledImageReader.getRGBImage(this, region, subsampling, getColorKeyMask());
+        return SampledImageReader.getRGBImage(this, region, subsampling, null);
     }
 
     @Override
@@ -370,7 +370,9 @@ public final class PDInlineImage implements PDImage
      * there is none.
      *
      * @return Mask Image XObject
+     * @deprecated inline images don't have a color key mask.
      */
+    @Deprecated
     public COSArray getColorKeyMask()
     {
         COSBase mask = parameters.getDictionaryObject(COSName.IM, COSName.MASK);
