@@ -70,7 +70,7 @@ public class PDExtendedGraphicsState implements COSObjectable
         {
             if( key.equals( COSName.LW ) )
             {
-                gs.setLineWidth( getLineWidth() );
+                gs.setLineWidth( defaultIfNull( getLineWidth(), 1 ) );
             }
             else if( key.equals( COSName.LC ) )
             {
@@ -82,7 +82,7 @@ public class PDExtendedGraphicsState implements COSObjectable
             }
             else if( key.equals( COSName.ML ) )
             {
-                gs.setMiterLimit( getMiterLimit() );
+                gs.setMiterLimit( defaultIfNull( getMiterLimit(), 10 ) );
             }
             else if( key.equals( COSName.D ) )
             {
@@ -94,7 +94,7 @@ public class PDExtendedGraphicsState implements COSObjectable
             }
             else if( key.equals( COSName.OPM ) )
             {
-                gs.setOverprintMode( getOverprintMode().doubleValue() );
+                gs.setOverprintMode( defaultIfNull( getOverprintMode(), 0 ) );
             }
             else if( key.equals( COSName.OP ) )
             {
@@ -115,11 +115,11 @@ public class PDExtendedGraphicsState implements COSObjectable
             }
             else if( key.equals( COSName.FL ) )
             {
-                gs.setFlatness( getFlatnessTolerance() );
+                gs.setFlatness( defaultIfNull( getFlatnessTolerance(), 1.0f ) );
             }
             else if( key.equals( COSName.SM ) )
             {
-                gs.setSmoothness( getSmoothnessTolerance() );
+                gs.setSmoothness( defaultIfNull( getSmoothnessTolerance(), 0 ) );
             }
             else if( key.equals( COSName.SA ) )
             {
@@ -127,11 +127,11 @@ public class PDExtendedGraphicsState implements COSObjectable
             }
             else if( key.equals( COSName.CA ) )
             {
-                gs.setAlphaConstant(getStrokingAlphaConstant());
+                gs.setAlphaConstant( defaultIfNull( getStrokingAlphaConstant(), 1.0f ) );
             }
             else if( key.equals( COSName.CA_NS ) )
             {
-                gs.setNonStrokeAlphaConstant(getNonStrokingAlphaConstant() );
+                gs.setNonStrokeAlphaConstant( defaultIfNull( getNonStrokingAlphaConstant(), 1.0f ) );
             }
             else if( key.equals( COSName.AIS ) )
             {
@@ -173,6 +173,22 @@ public class PDExtendedGraphicsState implements COSObjectable
         }
     }
 
+    /**
+     * Returns the provided default value in case 'standard' valu
+     * is <code>null</code>. To be used in cases unboxing may
+     * lead to a NPE.
+     *  
+     * @param _value  'standard' value
+     * @param _default  default value 
+     * 
+     * @return 'standard' value if not <code>null</code>
+     *         otherwise default value
+     */
+    private float defaultIfNull( Float _value, float _default ) 
+    {
+        return _value != null ? _value : _default;
+    }
+    
     /**
      * This will get the underlying dictionary that this class acts on.
      *
@@ -273,15 +289,15 @@ public class PDExtendedGraphicsState implements COSObjectable
     public PDLineDashPattern getLineDashPattern()
     {
         PDLineDashPattern retval = null;
-        COSArray dp = (COSArray) dict.getDictionaryObject( COSName.D );
-        if( dp != null )
+        COSBase dp = dict.getDictionaryObject( COSName.D );
+        if( dp instanceof COSArray  && ((COSArray)dp).size() == 2)
         {
-            COSArray array = new COSArray();
-            dp.addAll(dp);
-            dp.remove(dp.size() - 1);
-            int phase = dp.getInt(dp.size() - 1);
-
-            retval = new PDLineDashPattern( array, phase );
+            COSBase dashArray = ((COSArray)dp).getObject(0);
+            COSBase phase = ((COSArray)dp).getObject(1);
+            if (dashArray instanceof COSArray && phase instanceof COSNumber)
+            {
+                retval = new PDLineDashPattern((COSArray) dashArray, ((COSNumber) phase).intValue());
+            }
         }
         return retval;
     }
@@ -588,9 +604,10 @@ public class PDExtendedGraphicsState implements COSObjectable
     private Float getFloatItem( COSName key )
     {
         Float retval = null;
-        COSNumber value = (COSNumber) dict.getDictionaryObject( key );
-        if( value != null )
+        COSBase base = dict.getDictionaryObject(key);
+        if (base instanceof COSNumber)
         {
+            COSNumber value = (COSNumber) base;
             retval = value.floatValue();
         }
         return retval;
