@@ -411,6 +411,10 @@ class AppearanceGeneratorHelper
         
         // get the font
         PDFont font = defaultAppearance.getFont();
+        if (font == null)
+        {
+            throw new IllegalArgumentException("font is null, check whether /DA entry is incomplete or incorrect");
+        }
         
         // calculate the fontSize (because 0 = autosize)
         float fontSize = defaultAppearance.getFontSize();
@@ -424,7 +428,7 @@ class AppearanceGeneratorHelper
         // options
         if (field instanceof PDListBox)
         {
-            insertGeneratedSelectionHighlight(contents, appearanceStream, font, fontSize);
+            insertGeneratedListboxSelectionHighlight(contents, appearanceStream, font, fontSize);
         }
         
         // start the text output
@@ -609,7 +613,7 @@ class AppearanceGeneratorHelper
         }
     }
     
-    private void insertGeneratedSelectionHighlight(PDPageContentStream contents, PDAppearanceStream appearanceStream,
+    private void insertGeneratedListboxSelectionHighlight(PDPageContentStream contents, PDAppearanceStream appearanceStream,
             PDFont font, float fontSize) throws IOException
     {
         List<Integer> indexEntries = ((PDListBox) field).getSelectedOptionsIndex();
@@ -698,7 +702,7 @@ class AppearanceGeneratorHelper
             contents.newLineAtOffset(contentRect.getLowerLeftX(), yTextPos);
             contents.showText(options.get(i));
 
-            if (i - topIndex != (numOptions - 1))
+            if (i != (numOptions - 1))
             {
                 contents.endText();
             }
