@@ -251,6 +251,11 @@ public final class PDAcroForm implements COSObjectable
      */
     public void flatten(List<PDField> fields, boolean refreshAppearances) throws IOException
     {
+        // Nothing to flatten if there are no fields provided
+        if (fields.isEmpty()) {
+            return;
+        }
+
         // for dynamic XFA forms there is no flatten as this would mean to do a rendering
         // from the XFA content into a static PDF.
         if (xfaIsDynamic())
@@ -271,17 +276,21 @@ public final class PDAcroForm implements COSObjectable
         
         // the content stream to write to
         PDPageContentStream contentStream;
+
+        // get the widgets per page
+        Map<COSDictionary,Map<COSDictionary,PDAnnotationWidget>> pagesWidgetsMap = buildPagesWidgetsMap(fields);
         
         // preserve all non widget annotations
         for (PDPage page : document.getPages())
         {
+            Map<COSDictionary,PDAnnotationWidget> widgetsForPageMap = pagesWidgetsMap.get(page.getCOSObject());
             isContentStreamWrapped = false;
             
             List<PDAnnotation> annotations = new ArrayList<PDAnnotation>();
             
             for (PDAnnotation annotation: page.getAnnotations())
             {
-                if (!(annotation instanceof PDAnnotationWidget))
+                if (widgetsForPageMap != null && widgetsForPageMap.get(annotation.getCOSObject()) == null)
                 {
                     annotations.add(annotation);                 
                 }
@@ -350,7 +359,7 @@ public final class PDAcroForm implements COSObjectable
         }
         
         // remove the fields
-        setFields(Collections.<PDField>emptyList());
+        removeFields(fields);
         
         // remove XFA for hybrid forms
         dictionary.removeItem(COSName.XFA);
@@ -703,25 +712,6 @@ public final class PDAcroForm implements COSObjectable
         dictionary.setFlag(COSName.SIG_FLAGS, FLAG_APPEND_ONLY, appendOnly);
     }
     
-    private Map<COSDictionary, Integer> buildAnnotationToPageRef() {
-        Map<COSDictionary, Integer> annotationToPageRef = new HashMap<COSDictionary, Integer>();
-        
-        int idx = 0;
-        for (PDPage page : document.getPages()) {
-            try {
-                for (PDAnnotation annotation : page.getAnnotations()) {
-                    if (annotation instanceof PDAnnotationWidget) {
-                        annotationToPageRef.put(annotation.getCOSObject(), idx);
-                    }
-                }
-            } catch (IOException e) {
-                LOG.warn("Can't retriev annotations for page " + idx);
-            }
-            idx++;
-        }        
-        return annotationToPageRef;
-    }
-    
     /**
      * Check if there is a translation needed to place the annotations content.
      * 
@@ -730,7 +720,7 @@ public final class PDAcroForm implements COSObjectable
      */
     private boolean resolveNeedsTranslation(PDAppearanceStream appearanceStream)
     {
-        boolean needsTranslation = false;
+        boolean needsTranslation = true;
         
         PDResources resources = appearanceStream.getResources();
         if (resources != null && resources.getXObjectNames().iterator().hasNext())
@@ -750,9 +740,9 @@ public final class PDAcroForm implements COSObjectable
                         PDRectangle bbox = ((PDFormXObject)xObject).getBBox();
                         float llX = bbox.getLowerLeftX();
                         float llY = bbox.getLowerLeftY();
-                        if (llX == 0 && llY == 0)
+                        if (Float.compare(llX, 0) != 0 && Float.compare(llY, 0) != 0)
                         {
-                            needsTranslation = true;
+                            needsTranslation = false;
                         }
                     }
                 }
@@ -780,4 +770,75 @@ public final class PDAcroForm implements COSObjectable
         PDResources resources = appearanceStream.getResources();
         return resources != null && resources.getXObjectNames().iterator().hasNext();
     }
+
+    private Map<COSDictionary,Map<COSDictionary,PDAnnotationWidget>> buildPagesWidgetsMap(List<PDField> fields)
+    {
+        Map<COSDictionary,Map<COSDictionary,PDAnnotationWidget>> pagesAnnotationsMap = new HashMap<COSDictionary,Map<COSDictionary,PDAnnotationWidget>>();
+        boolean hasMissingPageRef = false;
+        
+        for (PDField field : fields)
+        {
+            List<PDAnnotationWidget> widgets = field.getWidgets();
+            for (PDAnnotationWidget widget : widgets)
+            {
+                PDPage pageForWidget = widget.getPage();
+                if (pageForWidget != null)
+                {
+                    if (pagesAnnotationsMap.get(pageForWidget.getCOSObject()) == null)
+                    {
+                        Map<COSDictionary,PDAnnotationWidget> widgetsForPage = new HashMap<COSDictionary,PDAnnotationWidget>();
+                        widgetsForPage.put(widget.getCOSObject(), widget);
+                        pagesAnnotationsMap.put(pageForWidget.getCOSObject(), widgetsForPage);
+                    }
+                    else
+                    {
+                        Map<COSDictionary,PDAnnotationWidget> widgetsForPage = pagesAnnotationsMap.get(pageForWidget.getCOSObject());
+                        widgetsForPage.put(widget.getCOSObject(), widget);
+                    }
+                }
+                else
+                {
+                    hasMissingPageRef = true;
+                }
+            }
+        }
+        
+        // TODO: if there is a widget with a missing page reference 
+        // we'd need to build the map reverse i.e. form the annotations to the 
+        // widget. But this will be much slower so will be omitted for now.
+        if (hasMissingPageRef)
+        {
+            LOG.warn("There has been a widget with a missing page reference. Please report to the PDFBox project");
+        }
+        
+        return pagesAnnotationsMap;
+    }
+    
+    private void removeFields(List<PDField> fields)
+    {
+        for (PDField field : fields) {
+            if (field.getParent() == null)
+            {
+                COSArray cosFields = (COSArray) dictionary.getDictionaryObject(COSName.FIELDS);
+                for (int i=0; i<cosFields.size(); i++)
+                {
+                    COSDictionary element = (COSDictionary) cosFields.getObject(i);
+                    if (field.getCOSObject().equals(element)) {
+                        cosFields.remove(i);
+                    }
+                }
+            }
+            else 
+            {
+                COSArray kids = (COSArray) field.getParent().getCOSObject().getDictionaryObject(COSName.KIDS);
+                for (int i=0; i<kids.size(); i++)
+                {
+                    COSDictionary element = (COSDictionary) kids.getObject(i);
+                    if (field.getCOSObject().equals(element)) {
+                        kids.remove(i);
+                    }
+                }
+            }
+        }        
+    }
 }
