@@ -209,6 +209,28 @@ public class TestSymmetricKeyEncryption extends TestCase
                 USERPASSWORD, OWNERPASSWORD, permission);
     }
 
+    /**
+     * PDFBOX-4308: test that index colorspace table string doesn't get
+     * corrupted when encrypting. This happened because the colorspace was
+     * referenced twice, once in the resources dictionary and once in an image
+     * in the resources dictionary, and when saving the PDF the string was saved
+     * twice, once as a direct object and once as an indirect object (both from
+     * the same java object). Encryption used the wrong object number and/or the
+     * object was encrypted twice.
+     *
+     * @throws IOException
+     */
+    public void testPDFBox4308() throws IOException
+    {
+        InputStream is = new FileInputStream("target/pdfs/PDFBOX-4308.pdf");
+        byte[] inputFileAsByteArray = IOUtils.toByteArray(is);
+        is.close();
+        int sizePriorToEncryption = inputFileAsByteArray.length;
+
+        testSymmEncrForKeySize(40, false, sizePriorToEncryption, inputFileAsByteArray,
+                USERPASSWORD, OWNERPASSWORD, permission);
+    }
+
     /**
      * Protect a document with an embedded PDF with a key and try to reopen it
      * with that key and compare.
