@@ -19,14 +19,15 @@ import java.util.EnumSet;
 import java.util.HashSet;
 import java.util.Set;
 import static org.junit.Assert.assertEquals;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.ExpectedException;
 
-/**
- * @author Tilman Hausherr
- */
 public class PageLayoutTest
 {
     /**
+     * @author Tilman Hausherr
+     *
      * Test for completeness (PDFBOX-3362).
      */
     @Test
@@ -43,4 +44,23 @@ public class PageLayoutTest
         assertEquals(PageLayout.values().length, pageLayoutSet.size());
         assertEquals(PageLayout.values().length, stringSet.size());
     }
+
+    /**
+     * @author John Bergqvist
+     */
+    @Rule
+    public ExpectedException thrown = ExpectedException.none();
+
+    @Test
+    public void fromStringInputNotNullOutputIllegalArgumentException()
+    {
+        // Arrange
+        final String value = "SinglePag";
+
+        // Act
+        thrown.expect(IllegalArgumentException.class);
+        PageLayout.fromString(value);
+
+        // Method is not expected to return due to exception thrown
+    }
 }
