@@ -20,6 +20,8 @@ package org.apache.pdfbox.pdmodel.font;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.util.HashSet;
+import java.util.Set;
 import junit.framework.TestCase;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -170,14 +172,56 @@ public class TestFontEmbedding extends TestCase
         assertEquals(expectedExtractedtext, extracted.replaceAll("\r", "").trim());
     }
 
+    /**
+     * Test corner case of PDFBOX-4302.
+     *
+     * @throws java.io.IOException
+     */
+    public void testMaxEntries() throws IOException
+    {
+        File file;
+        String text;
+        text = "あいうえおかきくけこさしすせそたちつてとなにぬねのはひふへほまみむめもやゆよらりるれろわをん" +
+                "アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヲン" +
+                "１２３４５６７８";
+
+        // The test must have MAX_ENTRIES_PER_OPERATOR unique characters
+        Set<Character> set = new HashSet<Character>(ToUnicodeWriter.MAX_ENTRIES_PER_OPERATOR);
+        for (int i = 0; i < text.length(); ++i)
+        {
+            set.add(text.charAt(i));
+        }
+        assertEquals(ToUnicodeWriter.MAX_ENTRIES_PER_OPERATOR, set.size());
+
+        PDDocument document = new PDDocument();
+        PDPage page = new PDPage(PDRectangle.A0);
+        document.addPage(page);
+        File ipafont = new File("target/fonts/ipag00303", "ipag.ttf");
+        PDType0Font font = PDType0Font.load(document, ipafont);
+        PDPageContentStream contentStream = new PDPageContentStream(document, page);
+        contentStream.beginText();
+        contentStream.setFont(font, 20);
+        contentStream.newLineAtOffset(50, 3000);
+        contentStream.showText(text);
+        contentStream.endText();
+        contentStream.close();
+        file = new File(OUT_DIR, "PDFBOX-4302-test.pdf");
+        document.save(file);
+        document.close();
+
+        // check that the extracted text matches what we wrote
+        String extracted = getUnicodeText(file);
+        assertEquals(text, extracted.trim());
+    }
+
     private void validateCIDFontType2(boolean useSubset) throws Exception
     {
         PDDocument document = new PDDocument();
         PDPage page = new PDPage(PDRectangle.A4);
         document.addPage(page);
 
-        InputStream input = TestFontEmbedding.class.getClassLoader().getResourceAsStream(
-                "org/apache/pdfbox/ttf/LiberationSans-Regular.ttf");
+        InputStream input = PDFont.class.getClassLoader().getResourceAsStream(
+                "org/apache/pdfbox/resources/ttf/LiberationSans-Regular.ttf");
         PDType0Font font = PDType0Font.load(document, input, useSubset);
 
         PDPageContentStream stream = new PDPageContentStream(document, page);
