@@ -216,7 +216,7 @@ public class PDButtonTest
     public void testAcrobatCheckBoxProperties() throws IOException
     {
         PDCheckBox checkbox = (PDCheckBox) acrobatAcroForm.getField("Checkbox");
-        assertEquals(checkbox.getValue(), "");
+        assertEquals(checkbox.getValue(), "Off");
         assertEquals(checkbox.isChecked(), false);
 
         checkbox.check();
@@ -260,7 +260,7 @@ public class PDButtonTest
     public void testAcrobatCheckBoxGroupProperties() throws IOException
     {
         PDCheckBox checkbox = (PDCheckBox) acrobatAcroForm.getField("CheckboxGroup");
-        assertEquals(checkbox.getValue(), "");
+        assertEquals(checkbox.getValue(), "Off");
         assertEquals(checkbox.isChecked(), false);
 
         checkbox.check();
