@@ -66,8 +66,8 @@ public interface PreflightConstants
     String OUTPUT_INTENT_DICTIONARY_KEY_OUTPUT_CONDITION_IDENTIFIER = "OutputConditionIdentifier";
     String OUTPUT_INTENT_DICTIONARY_VALUE_OUTPUT_CONDITION_IDENTIFIER_CUSTOM = "Custom";
 
-    String TRANPARENCY_DICTIONARY_KEY_EXTGSTATE = "ExtGState";
-    String TRANPARENCY_DICTIONARY_KEY_EXTGSTATE_ENTRY_REGEX = "(GS|gs)([0-9])+";
+    String TRANSPARENCY_DICTIONARY_KEY_EXTGSTATE = "ExtGState";
+    String TRANSPARENCY_DICTIONARY_KEY_EXTGSTATE_ENTRY_REGEX = "(GS|gs)([0-9])+";
 
     String TRANSPARENCY_DICTIONARY_KEY_BLEND_MODE = "BM";
     String TRANSPARENCY_DICTIONARY_KEY_UPPER_CA = "CA";
