@@ -178,9 +178,8 @@ public final class ExtractImages
                 throw new IOException("You do not have permission to extract images");
             }
 
-            for (int i = 0; i < document.getNumberOfPages(); i++) // todo: ITERATOR would be much better
+            for (PDPage page : document.getPages())
             {
-                PDPage page = document.getPage(i);
                 ImageGraphicsEngine extractor = new ImageGraphicsEngine(page);
                 extractor.run();
             }
@@ -214,6 +213,9 @@ public final class ExtractImages
                     PDTransparencyGroup group = softMask.getGroup();
                     if (group != null)
                     {
+                        // PDFBOX-4327: without this line NPEs will occur
+                        res.getExtGState(name).copyIntoGraphicsState(getGraphicsState());
+
                         processSoftMask(group);
                     }
                 }
