@@ -328,9 +328,11 @@ public class TextToPDF
                         throw new IOException( "Unknown argument: " + args[i] );
                     }
                 }
-                
-                app.createPDFFromText( doc, new FileReader( args[args.length-1] ) );
-                doc.save( args[args.length-2] );
+
+                FileReader fileReader = new FileReader(args[args.length - 1]);
+                app.createPDFFromText(doc, fileReader);
+                fileReader.close();
+                doc.save(args[args.length - 2]);
             }
         }
         finally
